searchNodes=[{"type":"module","title":"Flop","doc":"Flop is a helper library for filtering, ordering and pagination with Ecto. Usage The simplest way of using this library is just to use Flop.validate_and_run/3 and Flop.validate_and_run!/3 . Both functions take a queryable and a parameter map, validate the parameters, run the query and return the query results and the meta information. iex&gt; Flop.Repo . insert_all ( MyApp.Pet , [ ...&gt; %{ name : &quot;Harry&quot; , age : 4 , species : &quot;C. lupus&quot; } , ...&gt; %{ name : &quot;Maggie&quot; , age : 1 , species : &quot;O. cuniculus&quot; } , ...&gt; %{ name : &quot;Patty&quot; , age : 2 , species : &quot;C. aegagrus&quot; } ...&gt; ] ) iex&gt; params = %{ order_by : [ &quot;name&quot; , &quot;age&quot; ] , page : 1 , page_size : 2 } iex&gt; { :ok , { results , meta } } = ...&gt; Flop . validate_and_run ( ...&gt; MyApp.Pet , ...&gt; params , ...&gt; repo : Flop.Repo ...&gt; ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Harry&quot; , &quot;Maggie&quot; ] iex&gt; meta . total_count 3 iex&gt; meta . total_pages 2 iex&gt; meta . has_next_page? true Under the hood, these functions just call Flop.validate/2 and Flop.run/3 , which in turn calls Flop.all/3 and Flop.meta/3 . If you need finer control about if and when to execute each step, you can call those functions directly. See Flop.Meta for descriptions of the meta fields. Global configuration You can set some global options like the default Ecto repo via the application environment. All global options can be overridden by passing them directly to the functions or configuring the options for a schema module via Flop.Schema . import Config config :flop , repo : MyApp.Repo See Flop.option/0 for a description of all available options. Config modules Instead of setting global options in the application environment, you can also create a Flop config module. This is especially useful in an umbrella application, or if you have multiple Repos. defmodule MyApp.Flop do use Flop , repo : MyApp.Repo , default_limit : 25 end use Flop When you use Flop , the Flop module will define wrapper functions around all of the Flop functions that take a query, the Flop parameters, and options as arguments. The options passed to use Flop will be used as default options in all the wrapper functions, but you can still override them. The wrapped functions are: Flop.all/3 Flop.count/3 Flop.filter/3 Flop.meta/3 Flop.order_by/3 Flop.paginate/3 Flop.query/3 Flop.run/3 Flop.validate_and_run/3 Flop.validate_and_run!/3 So instead of using Flop.validate_and_run/3 , you would call MyApp.Flop.validate_and_run/3 . If you have both a config module and a global application config, Flop will fall back to the application config if an option is not set. See Flop.option/0 for a description of all available options. Schema options You can set some options for a schema by deriving Flop.Schema . The options are evaluated at the validation step. defmodule Pet do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , default_limit : 20 , max_limit : 100 } schema &quot;pets&quot; do field :name , :string field :age , :integer field :species , :string field :social_security_number , :string end end You need to pass the schema to Flop.validate/2 or any function that includes the validation step with the :for option. iex&gt; params = %{ &quot;order_by&quot; =&gt; [ &quot;name&quot; , &quot;age&quot; ] , &quot;limit&quot; =&gt; 5 } iex&gt; { :ok , flop } = Flop . validate ( params , for : MyApp.Pet ) iex&gt; flop . limit 5 iex&gt; params = %{ &quot;order_by&quot; =&gt; [ &quot;name&quot; , &quot;age&quot; ] , &quot;limit&quot; =&gt; 10_000 } iex&gt; { :error , meta } = Flop . validate ( params , for : MyApp.Pet ) iex&gt; [ limit : [ { msg , _ } ] ] = meta . errors iex&gt; msg &quot;must be less than or equal to %{number}&quot; iex&gt; params = %{ &quot;order_by&quot; =&gt; [ &quot;name&quot; , &quot;age&quot; ] , &quot;limit&quot; =&gt; 10_000 } iex&gt; { :error , % Flop.Meta { } = meta } = ...&gt; Flop . validate_and_run ( ...&gt; MyApp.Pet , ...&gt; params , ...&gt; for : MyApp.Pet ...&gt; ) iex&gt; [ limit : [ { msg , _ } ] ] = meta . errors iex&gt; msg &quot;must be less than or equal to %{number}&quot; Ordering To add an ordering clause to a query, you need to set the :order_by and optionally the :order_directions parameter. :order_by should be the list of fields, while :order_directions is a list of Flop.order_direction/0 . :order_by and :order_directions are zipped when generating the ORDER BY clause. If no order directions are given, :asc is used as default. iex&gt; params = %{ ...&gt; &quot;order_by&quot; =&gt; [ &quot;name&quot; , &quot;age&quot; ] , ...&gt; &quot;order_directions&quot; =&gt; [ &quot;asc&quot; , &quot;desc&quot; ] ...&gt; } iex&gt; { :ok , flop } = Flop . validate ( params ) iex&gt; flop . order_by [ :name , :age ] iex&gt; flop . order_directions [ :asc , :desc ] Flop uses these two fields instead of a keyword list, so that the order instructions can be easily passed in a query string. Pagination For queries using OFFSET and LIMIT , you have the choice between page-based pagination parameters: %{ page : 5 , page_size : 20 } and offset-based pagination parameters: %{ offset : 100 , limit : 20 } For cursor-based pagination, you can either use :first / :after or :last / :before . You also need to pass the :order_by parameter or set a default order for the schema via Flop.Schema . iex&gt; Flop.Repo . insert_all ( MyApp.Pet , [ ...&gt; %{ name : &quot;Harry&quot; , age : 4 , species : &quot;C. lupus&quot; } , ...&gt; %{ name : &quot;Maggie&quot; , age : 1 , species : &quot;O. cuniculus&quot; } , ...&gt; %{ name : &quot;Patty&quot; , age : 2 , species : &quot;C. aegagrus&quot; } ...&gt; ] ) iex&gt; iex&gt; # forward (first/after) iex&gt; iex&gt; params = %{ first : 2 , order_by : [ :species , :name ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( MyApp.Pet , params ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Patty&quot; , &quot;Harry&quot; ] iex&gt; meta . has_next_page? true iex&gt; end_cursor = meta . end_cursor &quot;g3QAAAACdwRuYW1lbQAAAAVIYXJyeXcHc3BlY2llc20AAAAIQy4gbHVwdXM=&quot; iex&gt; params = %{ first : 2 , after : end_cursor , order_by : [ :species , :name ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( MyApp.Pet , params ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Maggie&quot; ] iex&gt; meta . has_next_page? false iex&gt; iex&gt; # backward (last/before) iex&gt; iex&gt; params = %{ last : 2 , order_by : [ :species , :name ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( MyApp.Pet , params ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Harry&quot; , &quot;Maggie&quot; ] iex&gt; meta . has_previous_page? true iex&gt; start_cursor = meta . start_cursor &quot;g3QAAAACdwRuYW1lbQAAAAVIYXJyeXcHc3BlY2llc20AAAAIQy4gbHVwdXM=&quot; iex&gt; params = %{ last : 2 , before : start_cursor , order_by : [ :species , :name ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( MyApp.Pet , params ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Patty&quot; ] iex&gt; meta . has_previous_page? false By default, it is assumed that the query result is a list of maps or structs. If your query returns a different data structure, you can pass the :cursor_value_func option to retrieve the cursor values. See Flop.option/0 and Flop.Cursor for more information. You can restrict which pagination types are available. See Flop.option/0 for details. Filters Filters can be passed as a list of maps. It is recommended to define the filterable fields for a schema using Flop.Schema . iex&gt; Flop.Repo . insert_all ( MyApp.Pet , [ ...&gt; %{ name : &quot;Harry&quot; , age : 4 , species : &quot;C. lupus&quot; } , ...&gt; %{ name : &quot;Maggie&quot; , age : 1 , species : &quot;O. cuniculus&quot; } , ...&gt; %{ name : &quot;Patty&quot; , age : 2 , species : &quot;C. aegagrus&quot; } ...&gt; ] ) iex&gt; iex&gt; params = %{ filters : [ %{ field : :name , op : :=~ , value : &quot;Mag&quot; } ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( MyApp.Pet , params ) iex&gt; meta . total_count 1 iex&gt; [ pet ] = results iex&gt; pet . name &quot;Maggie&quot; See Flop.Filter.op/0 for a list of all available filter operators. GraphQL and Relay The parameters used for cursor-based pagination follow the Relay specification, so you can just pass the arguments you get from the client on to Flop. Flop.Relay can convert the query results returned by Flop.validate_and_run/3 into Edges and PageInfo formats required for Relay connections. For example, if you have a context module like this: defmodule MyApp.Flora import Ecto . query , warn : false alias MyApp.Flora.Plant def list_plants_by_continent ( % Continent { } = continent , %{ } = args ) do Plant |&gt; where ( continent_id : ^ continent . id ) |&gt; Flop . validate_and_run ( args , for : Plant ) end end Then your Absinthe resolver for the plants connection may look something like this: def list_plants ( args , %{ source : % Continent { } = continent } ) do with { :ok , result } &lt;- Flora . list_plants_by_continent ( continent , args ) do { :ok , Flop.Relay . connection_from_result ( result ) } end end","ref":"Flop.html"},{"type":"function","title":"Flop.aliases/2","doc":"Returns the names of the alias fields that are required for the order clause of the given Flop. The second argument is the schema module that derives Flop.Schema . For example, your schema module might define an alias field called :pet_count . @derive { Flop.Schema , filterable : [ ] , sortable : [ :name , :pet_count ] , alias_fields : [ :pet_count ] } If you pass a Flop that orders by the :pet_count field, the returned list will include the :pet_count alias. iex&gt; aliases ( % Flop { order_by : [ :name , :pet_count ] } , MyApp.Owner ) [ :pet_count ] If on the other hand only normal fields are used in the order_by parameter, an empty list will be returned. iex&gt; aliases ( % Flop { order_by : [ :name ] } , MyApp.Owner ) [ ] You can use this to dynamically build the select clause needed for the query. For more information about alias fields, refer to the module documentation of Flop.Schema .","ref":"Flop.html#aliases/2"},{"type":"function","title":"Flop.all/3","doc":"Applies the given Flop to the given queryable and returns all matchings entries. iex&gt; Flop . all ( MyApp.Pet , % Flop { } , repo : Flop.Repo ) [ ] You can also configure a default repo in your config files: config :flop , repo : MyApp.Repo This allows you to omit the third argument: iex&gt; Flop . all ( MyApp.Pet , % Flop { } , for : MyApp.Pet ) [ ] Note that when using cursor-based pagination, the applied limit will be first + 1 or last + 1 . The extra record is removed by Flop.run/3 , but not by this function. Also note that you will need to pass the for option in order for Flop to be able to find your join, compound, alias and custom field configuration. This function does not validate or apply default parameters to the given Flop struct. Be sure to validate any user-generated parameters with validate/2 or validate!/2 before passing them to this function.","ref":"Flop.html#all/3"},{"type":"function","title":"Flop.count/3","doc":"Returns the total count of entries matching the filter conditions of the Flop. The pagination and ordering option are disregarded. iex&gt; Flop . count ( MyApp.Pet , % Flop { } , repo : Flop.Repo ) 0 You can also configure a default repo in your config files: config :flop , repo : MyApp.Repo This allows you to omit the third argument: iex&gt; Flop . count ( MyApp.Pet , % Flop { } ) 0 You can override the default query by passing the :count_query option. This doesn't make a lot of sense when you use count/3 directly, but allows you to optimize the count query when you use one of the run/3 , validate_and_run/3 and validate_and_run!/3 functions. query = join ( Pet , :left , [ p ] , o in assoc ( p , :owner ) ) count_query = Pet count ( query , % Flop { } , count_query : count_query , for : Pet ) The filter parameters of the given Flop are applied to the custom count query. If for some reason you already have the count, you can pass it as the :count option. count ( query , % Flop { } , count : 42 , for : Pet ) If you pass both the :count and the :count_query options, the :count option will take precedence. This function does not validate or apply default parameters to the given Flop struct. Be sure to validate any user-generated parameters with validate/2 or validate!/2 before passing them to this function. Note that you will need to pass the for option in order for Flop to be able to find your join, compound, alias and custom field configuration.","ref":"Flop.html#count/3"},{"type":"function","title":"Flop.current_order/2","doc":"Returns the current order direction for the given field. Examples iex&gt; flop = % Flop { order_by : [ :name , :age ] , order_directions : [ :desc ] } iex&gt; current_order ( flop , :name ) :desc iex&gt; current_order ( flop , :age ) :asc iex&gt; current_order ( flop , :species ) nil","ref":"Flop.html#current_order/2"},{"type":"function","title":"Flop.filter/3","doc":"Applies the filter parameter of a Flop.t/0 to an Ecto.Queryable.t/0 . Used by Flop.query/2 . This function does not validate or apply default parameters to the given Flop struct. Be sure to validate any user-generated parameters with validate/2 or validate!/2 before passing them to this function. Note that you will need to pass the for option in order for Flop to be able to find your join, compound, alias and custom field configuration.","ref":"Flop.html#filter/3"},{"type":"function","title":"Flop.get_option/3","doc":"Returns the option with the given key. The look-up order is: the keyword list passed as the second argument the schema module that derives Flop.Schema , if the passed list includes the :for option the backend module with use Flop the application environment the default passed as the last argument","ref":"Flop.html#get_option/3"},{"type":"function","title":"Flop.map_to_filter_params/2","doc":"Converts a map of filter conditions into a list of Flop filter params. The default operator is :== . nil values are excluded from the result. iex&gt; params = map_to_filter_params ( ...&gt; %{ name : &quot;George&quot; , age : 8 , species : nil } ...&gt; ) iex&gt; Enum . sort ( params ) [ %{ field : :age , op : :== , value : 8 } , %{ field : :name , op : :== , value : &quot;George&quot; } ] iex&gt; params = map_to_filter_params ( ...&gt; %{ &quot;name&quot; =&gt; &quot;George&quot; , &quot;age&quot; =&gt; 8 , &quot;cat&quot; =&gt; true } ...&gt; ) iex&gt; Enum . sort ( params ) [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; 8 } , %{ &quot;field&quot; =&gt; &quot;cat&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; true } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;George&quot; } ] You can optionally pass a mapping from field names to operators as a map with atom keys. iex&gt; params = map_to_filter_params ( ...&gt; %{ name : &quot;George&quot; , age : 8 , species : nil } , ...&gt; operators : %{ name : :ilike_and } ...&gt; ) iex&gt; Enum . sort ( params ) [ %{ field : :age , op : :== , value : 8 } , %{ field : :name , op : :ilike_and , value : &quot;George&quot; } ] iex&gt; map_to_filter_params ( ...&gt; %{ &quot;name&quot; =&gt; &quot;George&quot; , &quot;age&quot; =&gt; 8 , &quot;cat&quot; =&gt; true } , ...&gt; operators : %{ name : :ilike_and , age : :&lt;= } ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; :&lt;= , &quot;value&quot; =&gt; 8 } , %{ &quot;field&quot; =&gt; &quot;cat&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; true } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; :ilike_and , &quot;value&quot; =&gt; &quot;George&quot; } ] You can also pass a map to rename fields. iex&gt; params = map_to_filter_params ( ...&gt; %{ s : &quot;George&quot; , age : 8 , species : nil } , ...&gt; rename : %{ s : :name } ...&gt; ) iex&gt; Enum . sort ( params ) [ %{ field : :age , op : :== , value : 8 } , %{ field : :name , op : :== , value : &quot;George&quot; } ] iex&gt; map_to_filter_params ( ...&gt; %{ &quot;s&quot; =&gt; &quot;George&quot; , &quot;cat&quot; =&gt; true } , ...&gt; rename : %{ s : :name , cat : :dog } ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;dog&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; true } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;George&quot; } ] If both a rename option and an operator are set for a field, the operator option needs to use the new field name. iex&gt; map_to_filter_params ( ...&gt; %{ n : &quot;George&quot; } , ...&gt; rename : %{ n : :name } , ...&gt; operators : %{ name : :ilike_or } ...&gt; ) [ %{ field : :name , op : :ilike_or , value : &quot;George&quot; } ] See also Flop.Filter.new/2 .","ref":"Flop.html#map_to_filter_params/2"},{"type":"function","title":"Flop.meta/3","doc":"Returns meta information for the given query and flop that can be used for building the pagination links. iex&gt; Flop . meta ( MyApp.Pet , % Flop { limit : 10 } , repo : Flop.Repo ) % Flop.Meta { current_offset : 0 , current_page : 1 , end_cursor : nil , flop : % Flop { limit : 10 } , has_next_page? : false , has_previous_page? : false , next_offset : nil , next_page : nil , opts : [ repo : Flop.Repo ] , page_size : 10 , previous_offset : nil , previous_page : nil , start_cursor : nil , total_count : 0 , total_pages : 0 } The function returns both the current offset and the current page, regardless of the pagination type. If the offset lies in between pages, the current page number is rounded up. This means that it is possible that the values for current_page and next_page can be identical. This can only occur if you use offset/limit based pagination with arbitrary offsets, but in that case, you will use the previous_offset , current_offset and next_offset values to render the pagination links anyway, so this shouldn't be a problem. Unless cursor-based pagination is used, this function will run a query to figure get the total count of matching records. This function does not validate or apply default parameters to the given Flop struct. Be sure to validate any user-generated parameters with validate/2 or validate!/2 before passing them to this function.","ref":"Flop.html#meta/3"},{"type":"function","title":"Flop.named_bindings/3","doc":"Returns the names of the bindings that are required for the filters and order clauses of the given Flop. The second argument is the schema module that derives Flop.Schema . For example, your schema module might define a join field called :owner_age . @derive { Flop.Schema , filterable : [ :name , :owner_age ] , sortable : [ :name , :owner_age ] , join_fields : [ owner_age : { :owner , :age } ] } If you pass a Flop with a filter on the :owner_age field, the returned list will include the :owner binding. iex&gt; named_bindings ( ...&gt; % Flop { ...&gt; filters : [ % Flop.Filter { field : :owner_age , op : :== , value : 5 } ] ...&gt; } , ...&gt; MyApp.Pet ...&gt; ) [ :owner ] If on the other hand only normal fields or compound fields are used in the filter and order options, or if the filter values are nil, an empty list will be returned. iex&gt; named_bindings ( ...&gt; % Flop { ...&gt; filters : [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;George&quot; } , ...&gt; % Flop.Filter { field : :owner_age , op : :== , value : nil } ...&gt; ] ...&gt; } , ...&gt; MyApp.Pet ...&gt; ) [ ] If a join field is part of a compound field, it will be returned. iex&gt; named_bindings ( ...&gt; % Flop { ...&gt; filters : [ ...&gt; % Flop.Filter { field : :pet_and_owner_name , op : :== , value : &quot;Mae&quot; } ...&gt; ] ...&gt; } , ...&gt; MyApp.Pet ...&gt; ) [ :owner ] For custom fields, you can set the :bindings option when you derive the Flop.Schema protocol. You can use this to dynamically build the join clauses needed for the query. See also Flop.with_named_bindings/4 . For more information about join fields, refer to the module documentation of Flop.Schema . Options :order - If false , only bindings needed for filtering are included. Defaults to true .","ref":"Flop.html#named_bindings/3"},{"type":"function","title":"Flop.nest_filters/3","doc":"Converts key/value filter parameters at the root of a map, converts them into a list of filter parameter maps and nests them under the :filters key. This is useful in cases where you get some or all filter parameters as key/value pairs instead of a full map with operators, for example when you expose certain filters with fixed operators on an API, or if you want to reflect some or all filters in the URL as path parameters or simple query parameters (e.g. /posts/{tag} or /posts?s=searchterm ). The given map should have either string keys or atom keys. Passing a map with mixed keys will lead to unexpected results and will cause an Ecto error when the return value is passed to one of the validation functions. The second argument is a list of fields as atoms. The opts argument is passed to map_to_filter_params/2 . The function returns a map that eventually needs to be passed to one of the Flop validation functions (any Flop.validate* function) before it can be used to make a query. Examples Map with atom keys iex&gt; nest_filters ( %{ name : &quot;Peter&quot; , page_size : 10 } , [ :name ] ) %{ filters : [ %{ field : :name , op : :== , value : &quot;Peter&quot; } ] , page_size : 10 } Map with string keys iex&gt; nest_filters ( %{ &quot;name&quot; =&gt; &quot;Peter&quot; } , [ :name ] ) %{ &quot;filters&quot; =&gt; [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;Peter&quot; } ] } Specifying operators iex&gt; nest_filters ( %{ name : &quot;Peter&quot; } , [ :name ] , operators : %{ name : :!= } ) %{ filters : [ %{ field : :name , op : :!= , value : &quot;Peter&quot; } ] } Renaming fields iex&gt; nest_filters ( %{ nombre : &quot;Peter&quot; , page_size : 10 } , [ :nombre ] , ...&gt; rename : %{ nombre : :name } ...&gt; ) %{ filters : [ %{ field : :name , op : :== , value : &quot;Peter&quot; } ] , page_size : 10 } iex&gt; nest_filters ( %{ &quot;nombre&quot; =&gt; &quot;Peter&quot; } , [ :nombre ] , ...&gt; rename : %{ nombre : :name } ...&gt; ) %{ &quot;filters&quot; =&gt; [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;Peter&quot; } ] } iex&gt; nest_filters ( %{ &quot;nombre&quot; =&gt; &quot;Peter&quot; } , [ :nombre ] , ...&gt; rename : %{ nombre : :name } , ...&gt; operators : %{ name : :like } ...&gt; ) %{ &quot;filters&quot; =&gt; [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; :like , &quot;value&quot; =&gt; &quot;Peter&quot; } ] } If the map already has a filters key, the extracted filters are added to the existing filters. iex&gt; nest_filters ( %{ name : &quot;Peter&quot; , filters : [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 25 } ] } , [ :name ] ) %{ filters : [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 25 } , %{ field : :name , op : :== , value : &quot;Peter&quot; } ] } iex&gt; nest_filters ( %{ &quot;name&quot; =&gt; &quot;Peter&quot; , &quot;filters&quot; =&gt; [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 25 } ] } , [ :name ] ) %{ &quot;filters&quot; =&gt; [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 25 } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;Peter&quot; } ] } If the existing filters are formatted as a map with integer indexes as keys as produced by a form, the map is converted to a list first. iex&gt; nest_filters ( %{ name : &quot;Peter&quot; , filters : %{ &quot;0&quot; =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 25 } } } , [ :name ] ) %{ filters : [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 25 } , %{ field : :name , op : :== , value : &quot;Peter&quot; } ] } iex&gt; nest_filters ( %{ &quot;name&quot; =&gt; &quot;Peter&quot; , &quot;filters&quot; =&gt; %{ &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 25 } } } , [ :name ] ) %{ &quot;filters&quot; =&gt; [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 25 } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;Peter&quot; } ] }","ref":"Flop.html#nest_filters/3"},{"type":"function","title":"Flop.order_by/3","doc":"Applies the order_by and order_directions parameters of a Flop.t/0 to an Ecto.Queryable.t/0 . Used by Flop.query/2 . This function does not validate or apply default parameters to the given Flop struct. Be sure to validate any user-generated parameters with validate/2 or validate!/2 before passing them to this function. Note that you will need to pass the for option in order for Flop to be able to find your join, compound, alias and custom field configuration.","ref":"Flop.html#order_by/3"},{"type":"function","title":"Flop.paginate/3","doc":"Applies the pagination parameters of a Flop.t/0 to an Ecto.Queryable.t/0 . The function supports both offset / limit based pagination and page / page_size based pagination. If you validated the Flop.t/0 with Flop.validate/1 before, you can be sure that the given Flop.t/0 only has pagination parameters set for one pagination method. If you pass an unvalidated Flop.t/0 that has pagination parameters set for multiple pagination methods, this function will arbitrarily only apply one of the pagination methods. Used by Flop.query/2 . This function does not validate or apply default parameters to the given Flop struct. Be sure to validate any user-generated parameters with validate/2 or validate!/2 before passing them to this function. Note that you will need to pass the for option in order for Flop to be able to find your join, compound, alias and custom field configuration.","ref":"Flop.html#paginate/3"},{"type":"function","title":"Flop.push_order/2","doc":"Updates the order_by and order_directions values of a Flop struct. If the field is not in the current order_by value, it will be prepended to the list. The order direction for the field will be set to :asc . If the field is already at the front of the order_by list, the order direction will be reversed. If the field is already in the list, but not at the front, it will be moved to the front and the order direction will be set to :asc . Example iex&gt; flop = push_order ( % Flop { } , :name ) iex&gt; flop . order_by [ :name ] iex&gt; flop . order_directions [ :asc ] iex&gt; flop = push_order ( flop , :age ) iex&gt; flop . order_by [ :age , :name ] iex&gt; flop . order_directions [ :asc , :asc ] iex&gt; flop = push_order ( flop , :age ) iex&gt; flop . order_by [ :age , :name ] iex&gt; flop . order_directions [ :desc , :asc ] iex&gt; flop = push_order ( flop , :species ) iex&gt; flop . order_by [ :species , :age , :name ] iex&gt; flop . order_directions [ :asc , :desc , :asc ] iex&gt; flop = push_order ( flop , :age ) iex&gt; flop . order_by [ :age , :species , :name ] iex&gt; flop . order_directions [ :asc , :asc , :asc ] If a string is passed as the second argument, it will be converted to an atom using String.to_existing_atom/1 . If the atom does not exist, the Flop struct will be returned unchanged. iex&gt; flop = push_order ( % Flop { } , &quot;name&quot; ) iex&gt; flop . order_by [ :name ] iex&gt; flop = push_order ( % Flop { } , &quot;this_atom_does_not_exist&quot; ) iex&gt; flop . order_by nil Since the pagination cursor depends on the sort order, the :before and :after parameters are reset. iex&gt; push_order ( % Flop { order_by : [ :id ] , after : &quot;ABC&quot; } , :name ) % Flop { order_by : [ :name , :id ] , order_directions : [ :asc ] , after : nil } iex&gt; push_order ( % Flop { order_by : [ :id ] , before : &quot;DEF&quot; } , :name ) % Flop { order_by : [ :name , :id ] , order_directions : [ :asc ] , before : nil }","ref":"Flop.html#push_order/2"},{"type":"function","title":"Flop.query/3","doc":"Adds clauses for filtering, ordering and pagination to a Ecto.Queryable.t/0 . The parameters are represented by the Flop.t/0 type. Any nil values will be ignored. This function does not validate or apply default parameters to the given Flop struct. Be sure to validate any user-generated parameters with validate/2 or validate!/2 before passing them to this function. Examples iex&gt; flop = % Flop { limit : 10 , offset : 19 } iex&gt; Flop . query ( MyApp.Pet , flop ) # Ecto.Query &lt; from p0 in MyApp.Pet , limit : ^ 10 , offset : ^ 19 &gt; Or enhance an already defined query: iex&gt; require Ecto.Query iex&gt; flop = % Flop { limit : 10 } iex&gt; q = Ecto.Query . where ( MyApp.Pet , species : &quot;dog&quot; ) iex&gt; Flop . query ( q , flop , for : MyApp.Pet ) # Ecto.Query &lt; from p0 in MyApp.Pet , where : p0 . species == &quot;dog&quot; , limit : ^ 10 &gt; Note that when using cursor-based pagination, the applied limit will be first + 1 or last + 1 . The extra record is removed by Flop.run/3 . Also note that you will need to pass the for option in order for Flop to be able to find your join, compound, alias and custom field configuration.","ref":"Flop.html#query/3"},{"type":"function","title":"Flop.reset_cursors/1","doc":"Removes the after and before cursors from a Flop struct. Example iex&gt; reset_cursors ( % Flop { after : &quot;A&quot; } ) % Flop { } iex&gt; reset_cursors ( % Flop { before : &quot;A&quot; } ) % Flop { }","ref":"Flop.html#reset_cursors/1"},{"type":"function","title":"Flop.reset_filters/1","doc":"Removes all filters from a Flop struct. Example iex&gt; reset_filters ( % Flop { filters : [ ...&gt; % Flop.Filter { field : :name , value : &quot;Jim&quot; } ...&gt; ] } ) % Flop { filters : [ ] }","ref":"Flop.html#reset_filters/1"},{"type":"function","title":"Flop.reset_order/1","doc":"Removes the order parameters from a Flop struct. Example iex&gt; reset_order ( % Flop { order_by : [ :name ] , order_directions : [ :asc ] } ) % Flop { order_by : nil , order_directions : nil }","ref":"Flop.html#reset_order/1"},{"type":"function","title":"Flop.run/3","doc":"Applies the given Flop to the given queryable, retrieves the data and the meta data. This function does not validate or apply default parameters to the given flop parameters. You can validate the parameters with Flop.validate/2 or Flop.validate!/2 , or you can use Flop.validate_and_run/3 or Flop.validate_and_run!/3 instead of this function. Note that you will need to pass the for option to both to validate/2 , validate!/2 and run/3 . Otherwise, Flop would not know how to find your field configuration (join fields, alias fields, custom fields, compound fields). iex&gt; opts = [ for : MyApp.Pet ] iex&gt; flop = Flop . validate! ( %{ } , opts ) iex&gt; { data , meta } = Flop . run ( MyApp.Pet , flop , opts ) iex&gt; data == [ ] true iex&gt; match? ( % Flop.Meta { } , meta ) true See the documentation for Flop.validate_and_run/3 for supported options.","ref":"Flop.html#run/3"},{"type":"function","title":"Flop.set_cursor/2","doc":"Takes a Flop.Meta struct and returns a Flop struct with updated cursor pagination params for going to either the previous or the next page. See to_previous_cursor/1 and to_next_cursor/1 for details. Examples iex&gt; set_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { first : 5 , after : &quot;a&quot; } , ...&gt; has_previous_page? : true , start_cursor : &quot;b&quot; ...&gt; } , ...&gt; :previous ...&gt; ) % Flop { last : 5 , before : &quot;b&quot; } iex&gt; set_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { first : 5 , after : &quot;a&quot; } , ...&gt; has_next_page? : true , end_cursor : &quot;b&quot; ...&gt; } , ...&gt; :next ...&gt; ) % Flop { first : 5 , after : &quot;b&quot; }","ref":"Flop.html#set_cursor/2"},{"type":"function","title":"Flop.set_offset/2","doc":"Sets the offset value of a Flop struct while also removing/converting pagination parameters for other pagination types. iex&gt; set_offset ( % Flop { limit : 10 , offset : 10 } , 20 ) % Flop { offset : 20 , limit : 10 } iex&gt; set_offset ( % Flop { page : 5 , page_size : 10 } , 20 ) % Flop { limit : 10 , offset : 20 , page : nil , page_size : nil } iex&gt; set_offset ( % Flop { limit : 10 , offset : 10 } , &quot;20&quot; ) % Flop { offset : 20 , limit : 10 } The offset will not be allowed to go below 0. iex&gt; set_offset ( % Flop { } , - 5 ) % Flop { offset : 0 }","ref":"Flop.html#set_offset/2"},{"type":"function","title":"Flop.set_page/2","doc":"Sets the page value of a Flop struct while also removing/converting pagination parameters for other pagination types. iex&gt; set_page ( % Flop { page : 2 , page_size : 10 } , 6 ) % Flop { page : 6 , page_size : 10 } iex&gt; set_page ( % Flop { limit : 10 , offset : 20 } , 8 ) % Flop { limit : nil , offset : nil , page : 8 , page_size : 10 } iex&gt; set_page ( % Flop { page : 2 , page_size : 10 } , &quot;6&quot; ) % Flop { page : 6 , page_size : 10 } The page number will not be allowed to go below 1. iex&gt; set_page ( % Flop { } , - 5 ) % Flop { page : 1 }","ref":"Flop.html#set_page/2"},{"type":"function","title":"Flop.to_next_cursor/1","doc":"Takes a Flop.Meta struct and returns a Flop struct with updated cursor pagination params for going to the next page. Examples iex&gt; to_next_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { first : 5 , after : &quot;a&quot; } , ...&gt; has_next_page? : true , end_cursor : &quot;b&quot; ...&gt; } ...&gt; ) % Flop { first : 5 , after : &quot;b&quot; } iex&gt; to_next_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { last : 5 , before : &quot;b&quot; } , ...&gt; has_next_page? : true , end_cursor : &quot;a&quot; ...&gt; } ...&gt; ) % Flop { first : 5 , after : &quot;a&quot; } If there is no next page, the Flop struct is returned unchanged. iex&gt; to_next_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { first : 5 , after : &quot;a&quot; } , ...&gt; has_next_page? : false , start_cursor : &quot;b&quot; ...&gt; } ...&gt; ) % Flop { first : 5 , after : &quot;a&quot; } It can happen that has_next_page? is true , but no end cursor is set, for example if the user is on the second page and switches to the previous page right after the only item on the first page is deleted. Since the result set is empty and there is no end cursor in this case, this function sets the new after parameter to nil . By doing this, the user is sent to the first page without skipping any items. iex&gt; to_next_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { first : 5 , after : &quot;a&quot; } , ...&gt; has_next_page? : true , end_cursor : nil ...&gt; } ...&gt; ) % Flop { first : 5 , after : nil } iex&gt; to_next_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { last : 5 , before : &quot;b&quot; } , ...&gt; has_next_page? : true , end_cursor : nil ...&gt; } ...&gt; ) % Flop { first : 5 , after : nil }","ref":"Flop.html#to_next_cursor/1"},{"type":"function","title":"Flop.to_next_offset/2","doc":"Sets the offset of a Flop struct to the next page depending on the limit. If the total count is given as the second argument, the offset will not be increased if the last page has already been reached. You can get the total count from the Flop.Meta struct. If the Flop has an offset beyond the total count, the offset will be set to the last page. Examples iex&gt; to_next_offset ( % Flop { offset : 10 , limit : 5 } ) % Flop { offset : 15 , limit : 5 } iex&gt; to_next_offset ( % Flop { offset : 15 , limit : 5 } , 21 ) % Flop { offset : 20 , limit : 5 } iex&gt; to_next_offset ( % Flop { offset : 15 , limit : 5 } , 20 ) % Flop { offset : 15 , limit : 5 } iex&gt; to_next_offset ( % Flop { offset : 28 , limit : 5 } , 22 ) % Flop { offset : 20 , limit : 5 } iex&gt; to_next_offset ( % Flop { offset : - 5 , limit : 20 } ) % Flop { offset : 0 , limit : 20 }","ref":"Flop.html#to_next_offset/2"},{"type":"function","title":"Flop.to_next_page/2","doc":"Sets the page of a Flop struct to the next page. If the total number of pages is given as the second argument, the page number will not be increased if the last page has already been reached. You can get the total number of pages from the Flop.Meta struct. Examples iex&gt; to_next_page ( % Flop { page : 5 } ) % Flop { page : 6 } iex&gt; to_next_page ( % Flop { page : 5 } , 6 ) % Flop { page : 6 } iex&gt; to_next_page ( % Flop { page : 6 } , 6 ) % Flop { page : 6 } iex&gt; to_next_page ( % Flop { page : 7 } , 6 ) % Flop { page : 6 } iex&gt; to_next_page ( % Flop { page : - 5 } ) % Flop { page : 1 }","ref":"Flop.html#to_next_page/2"},{"type":"function","title":"Flop.to_previous_cursor/1","doc":"Takes a Flop.Meta struct and returns a Flop struct with updated cursor pagination params for going to the previous page. Examples iex&gt; to_previous_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { first : 5 , after : &quot;a&quot; } , ...&gt; has_previous_page? : true , start_cursor : &quot;b&quot; ...&gt; } ...&gt; ) % Flop { last : 5 , before : &quot;b&quot; } iex&gt; to_previous_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { last : 5 , before : &quot;b&quot; } , ...&gt; has_previous_page? : true , start_cursor : &quot;a&quot; ...&gt; } ...&gt; ) % Flop { last : 5 , before : &quot;a&quot; } If there is no previous page, the Flop struct is returned unchanged. iex&gt; to_previous_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { first : 5 , after : &quot;b&quot; } , ...&gt; has_previous_page? : false , start_cursor : &quot;a&quot; ...&gt; } ...&gt; ) % Flop { first : 5 , after : &quot;b&quot; } It can happen that has_previous_page? is true , but no start cursor is set, for example if the user is on the next-to-last page and switches to the next page right after the only item on the last page is deleted. Since the result set would be empty and there is no start cursor in this case, this function sets the new before parameter to nil . By doing this, the user is sent to the actual last page without skipping any items. iex&gt; to_previous_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { first : 5 , after : &quot;a&quot; } , ...&gt; has_previous_page? : true , start_cursor : nil ...&gt; } ...&gt; ) % Flop { last : 5 , before : nil } iex&gt; to_previous_cursor ( ...&gt; % Flop.Meta { ...&gt; flop : % Flop { last : 5 , before : &quot;b&quot; } , ...&gt; has_previous_page? : true , start_cursor : nil ...&gt; } ...&gt; ) % Flop { last : 5 , before : nil }","ref":"Flop.html#to_previous_cursor/1"},{"type":"function","title":"Flop.to_previous_offset/1","doc":"Sets the offset of a Flop struct to the page depending on the limit. Examples iex&gt; to_previous_offset ( % Flop { offset : 20 , limit : 10 } ) % Flop { offset : 10 , limit : 10 } iex&gt; to_previous_offset ( % Flop { offset : 5 , limit : 10 } ) % Flop { offset : 0 , limit : 10 } iex&gt; to_previous_offset ( % Flop { offset : 0 , limit : 10 } ) % Flop { offset : 0 , limit : 10 } iex&gt; to_previous_offset ( % Flop { offset : - 2 , limit : 10 } ) % Flop { offset : 0 , limit : 10 }","ref":"Flop.html#to_previous_offset/1"},{"type":"function","title":"Flop.to_previous_page/1","doc":"Sets the page of a Flop struct to the previous page, but not less than 1. Examples iex&gt; to_previous_page ( % Flop { page : 5 } ) % Flop { page : 4 } iex&gt; to_previous_page ( % Flop { page : 1 } ) % Flop { page : 1 } iex&gt; to_previous_page ( % Flop { page : - 2 } ) % Flop { page : 1 }","ref":"Flop.html#to_previous_page/1"},{"type":"function","title":"Flop.unnest_filters/3","doc":"Takes a Flop, converts it to a map and unnests the filters for the given fields. This is the reverse operation of nest_filters/3 , with the caveat that the result of nest_filters/3 needs to be validated to convert it to a Flop struct before it can be passed back to unnest_filters/3 . The function returns a map with nil values removed. Examples iex&gt; unnest_filters ( ...&gt; % Flop { ...&gt; filters : [ % Flop.Filter { field : :name , op : :== , value : &quot;Peter&quot; } ] , ...&gt; page_size : 10 ...&gt; } , ...&gt; [ :name ] ...&gt; ) %{ name : &quot;Peter&quot; , page_size : 10 } iex&gt; unnest_filters ( ...&gt; % Flop { ...&gt; filters : [ % Flop.Filter { field : :name , op : :== , value : nil } ] , ...&gt; page_size : 10 ...&gt; } , ...&gt; [ :name ] ...&gt; ) %{ page_size : 10 } To rename fields, you can pass a map, where the keys are the field names in the unnested map and the values are the field names in the Flop.Filter struct. You can pass the exact same rename map to nest_filters/3 and unnest_filters/3 to revert the nesting/unnesting. iex&gt; unnest_filters ( ...&gt; % Flop { ...&gt; filters : [ % Flop.Filter { field : :name , op : :== , value : &quot;Peter&quot; } ] , ...&gt; page_size : 10 ...&gt; } , ...&gt; [ :name ] , ...&gt; rename : %{ nombre : :name } ...&gt; ) %{ nombre : &quot;Peter&quot; , page_size : 10 }","ref":"Flop.html#unnest_filters/3"},{"type":"function","title":"Flop.validate/2","doc":"Validates a Flop.t/0 . Examples iex&gt; params = %{ &quot;limit&quot; =&gt; 10 , &quot;offset&quot; =&gt; 0 , &quot;texture&quot; =&gt; &quot;fluffy&quot; } iex&gt; Flop . validate ( params ) { :ok , % Flop { filters : [ ] , limit : 10 , offset : 0 , order_by : nil , order_directions : nil , page : nil , page_size : nil } } iex&gt; flop = % Flop { offset : - 1 } iex&gt; { :error , % Flop.Meta { } = meta } = Flop . validate ( flop ) iex&gt; meta . errors [ offset : [ { &quot;must be greater than or equal to %{number}&quot; , [ validation : :number , kind : :greater_than_or_equal_to , number : 0 ] } ] ] It also makes sure that only one pagination method is used. iex&gt; params = %{ limit : 10 , offset : 0 , page : 5 , page_size : 10 } iex&gt; { :error , % Flop.Meta { } = meta } = Flop . validate ( params ) iex&gt; meta . errors [ limit : [ { &quot;cannot combine multiple pagination types&quot; , [ ] } ] ] If you derived Flop.Schema in your Ecto schema to define the filterable and sortable fields, you can pass the module name to the function to validate that only allowed fields are used. The function will also apply any default values set for the schema. iex&gt; params = %{ &quot;order_by&quot; =&gt; [ &quot;species&quot; ] } iex&gt; { :error , % Flop.Meta { } = meta } = Flop . validate ( params , for : MyApp.Pet ) iex&gt; [ order_by : [ { msg , [ _ , { _ , enum } ] } ] ] = meta . errors iex&gt; msg &quot;has an invalid entry&quot; iex&gt; enum [ :name , :age , :owner_name , :owner_age ] Note that currently, trying to use an existing field that is not allowed as seen above will result in the error message has an invalid entry , while trying to use a field name that does not exist in the schema (or more precisely: a field name that doesn't exist as an atom) will result in the error message is invalid . This might change in the future.","ref":"Flop.html#validate/2"},{"type":"function","title":"Flop.validate!/2","doc":"Same as Flop.validate/2 , but raises an Ecto.InvalidChangesetError if the parameters are invalid.","ref":"Flop.html#validate!/2"},{"type":"function","title":"Flop.validate_and_run/3","doc":"Validates the given flop parameters and retrieves the data and meta data on success. iex&gt; { :ok , { [ ] , % Flop.Meta { } } } = ...&gt; Flop . validate_and_run ( MyApp.Pet , % Flop { } , for : MyApp.Pet ) iex&gt; { :error , % Flop.Meta { } = meta } = ...&gt; Flop . validate_and_run ( MyApp.Pet , % Flop { limit : - 1 } ) iex&gt; meta . errors [ limit : [ { &quot;must be greater than %{number}&quot; , [ validation : :number , kind : :greater_than , number : 0 ] } ] ] Options for : Passed to Flop.validate/2 . Also used to look up the join, alias, compound and custom field configuration. repo : The Ecto.Repo module. Required if no default repo is configured. cursor_value_func : An arity-2 function to be used to retrieve an unencoded cursor value from a query result item and the order_by fields. Defaults to Flop.Cursor.get_cursor_from_node/2 . count_query : Lets you override the base query for counting, e.g. if you don't want to include unnecessary joins. The filter parameters are applied to the given query. See also Flop.count/3 .","ref":"Flop.html#validate_and_run/3"},{"type":"function","title":"Flop.validate_and_run!/3","doc":"Same as Flop.validate_and_run/3 , but raises on error.","ref":"Flop.html#validate_and_run!/3"},{"type":"function","title":"Flop.with_named_bindings/4","doc":"Applies a callback function to a query for all named bindings that are necessary for the given Flop parameters. The callback function must accept a queryable and the name of the binding and return a query that includes the given binding. This is the same as Ecto.Query.with_named_binding/3 , except that the callback function must take the second argument. Options :for (required) - The schema module that derives Flop.Schema . :order - If false , only bindings needed for filtering are included. Defaults to true . Example def list_pets ( params ) do opts = [ for : Pet ] with { :ok , flop } &lt;- Flop . validate ( params , opts ) do Pet |&gt; Flop . with_named_bindings ( flop , &amp; join_pet_assocs / 2 , opts ) |&gt; Flop . run ( flop , opts ) end end defp join_pet_assocs ( query , :owner ) do join ( query , [ p ] , o in assoc ( p , :owner ) , as : :owner ) end defp join_pet_assocs ( query , :toys ) do join ( query , [ p ] , t in assoc ( p , :toys ) , as : :toys ) end Since the callback function has the same arguments and return value as the one passed to Ecto.Query.with_named_binding/3 , you can reuse the function to add any other bindings you may need for the query besides the ones for Flop filters. This also means you can use Ecto.Query.with_named_binding/3 to recursively add bindings in case you need intermediate joins to get to a nested association. def list_owners ( params ) do opts = [ for : Owner ] with { :ok , flop } &lt;- Flop . validate ( params , opts ) do Pet |&gt; Flop . with_named_bindings ( flop , &amp; join_owner_assocs / 2 , opts ) |&gt; Flop . run ( flop , opts ) end end defp join_owner_assocs ( query , :pets ) do join ( query , [ o ] , p in assoc ( o , :pets ) , as : :pets ) end defp join_owner_assocs ( query , :toys ) do query |&gt; Ecto.Query . with_named_binding ( :pets , &amp; join_owner_assocs / 2 ) |&gt; join ( query , [ pets : p ] , t in assoc ( p , :toys ) , as : :toys ) end","ref":"Flop.html#with_named_bindings/4"},{"type":"type","title":"Flop.default_order/0","doc":"","ref":"Flop.html#t:default_order/0"},{"type":"type","title":"Flop.option/0","doc":"Options that can be passed to most of the functions or that can be set via the application environment. :cursor_value_func - 2-arity function used to get the (unencoded) cursor value from a record. Only used with cursor-based pagination. The first argument is the record, the second argument is the list of fields used in the ORDER BY clause. Needs to return a map with the order fields as keys and the the record values of these fields as values. Defaults to Flop.Cursor.get_cursor_from_node/2 . :default_limit - Sets a global default limit for queries that is used if no default limit is set for a schema and no limit is set in the parameters. Set to false to not set any default limit. Defaults to 50 . :default_order - Sets the default order for a query if none is passed in the parameters or if ordering is disabled. Can be set in the schema or in the options passed to the query functions. :default_pagination_type - The pagination type to use when setting default parameters and the pagination type cannot be determined from the parameters. Parameters for other pagination types can still be passed when setting this option. To restrict which pagination types can be used, set the :pagination_types option. Set to false to override a default. :filtering (boolean) - Can be set to false to silently ignore filter parameters. :for - The schema module to be used for validation. Flop.Schema must be derived for the given module. This option is optional and can not be set globally. If it is not set, schema specific validation will be omitted. Used by the validation functions. It is also used to determine which fields are join and compound fields. :max_limit - Sets a global maximum limit for queries that is used if no maximum limit is set for a schema. Set to false to not set any max limit. Defaults to 1000 . :count_query - Allows you to set a separate base query for counting. Can only be passed as an option to one of the query functions. See Flop.validate_and_run/3 and Flop.count/3 . :count - Allows you to set the count, useful for when the count is already computed elsewhere. Can only be passed as an option to one of the query functions. See Flop.validate_and_run/3 and Flop.count/3 . :pagination (boolean) - Can be set to false to silently ignore pagination parameters. :pagination_types - Defines which pagination types are allowed. Parameters for other pagination types will not be cast. By default, all pagination types are allowed. See also Flop.pagination_type/0 . :query_opts - These options are passed to the Ecto.Repo query functions. See the Ecto documentation for Ecto.Repo.all/2 , Ecto.Repo.aggregate/3 , and the &quot;Shared Options&quot; section of Ecto.Repo . :ordering (boolean) - Can be set to false to silently ignore order parameters. Default orders are still applied. :repo - The Ecto Repo module to use for the database query. Used by all functions that execute a database query. :replace_invalid_params - If true , invalid parameters will be replaced with default values if possible or removed. Defaults to false . extra_opts (keyword list) - Additional options passed through to custom fields All options can be passed directly to the functions. Some of the options can be set on a schema level via Flop.Schema . All options except :for , :default_order and :count_query can be set globally via the application environment. import Config config :flop , default_limit : 25 , filtering : false , cursor_value_func : &amp; MyApp.Repo . get_cursor_value / 2 , max_limit : 100 , ordering : false , pagination_types : [ :first , :last , :page ] , repo : MyApp.Repo , query_opts : [ prefix : &quot;some-prefix&quot; ] The look up order is: option passed to function option set for schema using Flop.Schema (only :max_limit , :default_limit , :default_order , :pagination_types and :default_pagination_type ) option set in config module, if one is used (except :for , :default_order and :count_query ; see section &quot;Config modules&quot; in the module documentation) option set in global config (except :for , :default_order and :count_query ) default value (only :cursor_value_func )","ref":"Flop.html#t:option/0"},{"type":"type","title":"Flop.order_direction/0","doc":"Represents the supported order direction values.","ref":"Flop.html#t:order_direction/0"},{"type":"type","title":"Flop.pagination_type/0","doc":"Represents the pagination type. :offset - pagination using the offset and limit parameters :page - pagination using the page and page_size parameters :first - cursor-based pagination using the first and after parameters :last - cursor-based pagination using the last and before parameters","ref":"Flop.html#t:pagination_type/0"},{"type":"type","title":"Flop.t/0","doc":"Represents the query parameters for filtering, ordering and pagination. Fields after : Used for cursor-based pagination. Must be used with first or a default limit. before : Used for cursor-based pagination. Must be used with last or a default limit. decoded_cursor : Used internally to hold on to the decoded cursor between validation and query execution. Value is discarded when meta is built. limit , offset : Used for offset-based pagination. first Used for cursor-based pagination. Can be used alone to begin pagination or with after last Used for cursor-based pagination. page , page_size : Used for offset-based pagination as an alternative to offset and limit . order_by : List of fields to order by. Fields can be restricted by deriving Flop.Schema in your Ecto schema. order_directions : List of order directions applied to the fields defined in order_by . If empty or the list is shorter than the order_by list, :asc will be used as a default for each missing order direction. filters : List of filters, see Flop.Filter.t/0 .","ref":"Flop.html#t:t/0"},{"type":"module","title":"Flop.Cursor","doc":"Functions for encoding, decoding and extracting cursor values.","ref":"Flop.Cursor.html"},{"type":"function","title":"Flop.Cursor.decode/1","doc":"Decodes a cursor value. Returns :error if the cursor cannot be decoded or the decoded term is not a map with atom keys. iex&gt; Flop.Cursor . decode ( &quot;g3QAAAABZAACaWRiAAACDg==&quot; ) { :ok , %{ id : 526 } } iex&gt; Flop.Cursor . decode ( &quot;AAAH&quot; ) :error iex&gt; f = fn a -&gt; a + 1 end iex&gt; cursor = Flop.Cursor . encode ( %{ a : f } ) iex&gt; Flop.Cursor . decode ( cursor ) :error iex&gt; cursor = Flop.Cursor . encode ( a : &quot;b&quot; ) iex&gt; Flop.Cursor . decode ( cursor ) :error iex&gt; cursor = Flop.Cursor . encode ( %{ &quot;a&quot; =&gt; &quot;b&quot; } ) iex&gt; Flop.Cursor . decode ( cursor ) :error Trying to decode a cursor that contains non-existent atoms also results in an error. iex&gt; Flop.Cursor . decode ( &quot;g3QAAAABZAAGYmFybmV5ZAAGcnViYmVs&quot; ) :error","ref":"Flop.Cursor.html#decode/1"},{"type":"function","title":"Flop.Cursor.decode!/1","doc":"Same as Flop.Cursor.decode/1 , but raises an error if the cursor is invalid. iex&gt; Flop.Cursor . decode! ( &quot;g3QAAAABZAACaWRiAAACDg==&quot; ) %{ id : 526 } iex&gt; Flop.Cursor . decode! ( &quot;AAAH&quot; ) ** (RuntimeError) invalid cursor","ref":"Flop.Cursor.html#decode!/1"},{"type":"function","title":"Flop.Cursor.encode/1","doc":"Encodes a cursor value. Flop.Cursor . encode ( %{ email : &quot;peter@mail&quot; , name : &quot;Peter&quot; } ) &quot;g3QAAAACdwRuYW1lbQAAAAVQZXRlcncFZW1haWxtAAAACnBldGVyQG1haWw=&quot;","ref":"Flop.Cursor.html#encode/1"},{"type":"function","title":"Flop.Cursor.get_cursor_from_edge/2","doc":"Takes a tuple with the node and the edge and the order_by field list and returns the cursor value derived from the edge map. If a map is passed instead of a tuple, it retrieves the cursor value from that map. This function can be used for the :cursor_value_func option. See also Flop.Cursor.get_cursor_from_node/2 . iex&gt; record = %{ id : 20 , name : &quot;George&quot; , age : 62 } iex&gt; edge = %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; iex&gt; Flop.Cursor . get_cursor_from_edge ( { record , edge } , [ :id ] ) %{ id : 25 } iex&gt; Flop.Cursor . get_cursor_from_edge ( { record , edge } , [ :id , :relation ] ) %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; Flop.Cursor . get_cursor_from_edge ( record , [ :id ] ) %{ id : 20 } If the edge is a struct that derives Flop.Schema , join and compound fields are resolved according to the configuration. iex&gt; record = %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; edge = % MyApp.Pet { ...&gt; name : &quot;George&quot; , ...&gt; owner : % MyApp.Owner { name : &quot;Carl&quot; } ...&gt; } iex&gt; iex&gt; Flop.Cursor . get_cursor_from_edge ( { record , edge } , [ :owner_name ] ) %{ owner_name : &quot;Carl&quot; } iex&gt; Flop.Cursor . get_cursor_from_edge ( edge , [ :owner_name ] ) %{ owner_name : &quot;Carl&quot; } iex&gt; record = %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; edge = % MyApp.Pet { ...&gt; given_name : &quot;George&quot; , ...&gt; family_name : &quot;Gooney&quot; ...&gt; } iex&gt; Flop.Cursor . get_cursor_from_edge ( { record , edge } , [ :full_name ] ) %{ full_name : &quot;Gooney George&quot; } iex&gt; Flop.Cursor . get_cursor_from_edge ( edge , [ :full_name ] ) %{ full_name : &quot;Gooney George&quot; }","ref":"Flop.Cursor.html#get_cursor_from_edge/2"},{"type":"function","title":"Flop.Cursor.get_cursor_from_node/2","doc":"Takes a tuple with the node and the edge and the order_by field list and returns the cursor value derived from the node map. If a map is passed instead of a tuple, it retrieves the cursor value from that map. This function is used as a default if no :cursor_value_func option is set. See also Flop.Cursor.get_cursor_from_edge/2 . iex&gt; record = %{ id : 20 , name : &quot;George&quot; , age : 62 } iex&gt; edge = %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; iex&gt; Flop.Cursor . get_cursor_from_node ( { record , edge } , [ :id ] ) %{ id : 20 } iex&gt; Flop.Cursor . get_cursor_from_node ( { record , edge } , [ :id , :name ] ) %{ id : 20 , name : &quot;George&quot; } iex&gt; Flop.Cursor . get_cursor_from_node ( record , [ :id ] ) %{ id : 20 } If the node is a struct that derives Flop.Schema , join and compound fields are resolved according to the configuration. iex&gt; record = % MyApp.Pet { ...&gt; name : &quot;George&quot; , ...&gt; owner : % MyApp.Owner { name : &quot;Carl&quot; } ...&gt; } iex&gt; edge = %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; iex&gt; Flop.Cursor . get_cursor_from_node ( { record , edge } , [ :owner_name ] ) %{ owner_name : &quot;Carl&quot; } iex&gt; Flop.Cursor . get_cursor_from_node ( record , [ :owner_name ] ) %{ owner_name : &quot;Carl&quot; } iex&gt; record = % MyApp.Pet { ...&gt; given_name : &quot;George&quot; , ...&gt; family_name : &quot;Gooney&quot; ...&gt; } iex&gt; edge = %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; Flop.Cursor . get_cursor_from_node ( { record , edge } , [ :full_name ] ) %{ full_name : &quot;Gooney George&quot; } iex&gt; Flop.Cursor . get_cursor_from_node ( record , [ :full_name ] ) %{ full_name : &quot;Gooney George&quot; }","ref":"Flop.Cursor.html#get_cursor_from_node/2"},{"type":"function","title":"Flop.Cursor.get_cursors/3","doc":"Retrieves the start and end cursors from a query result. iex&gt; results = [ %{ name : &quot;Mary&quot; } , %{ name : &quot;Paul&quot; } , %{ name : &quot;Peter&quot; } ] iex&gt; order_by = [ :name ] iex&gt; iex&gt; { start_cursor , end_cursor } = ...&gt; Flop.Cursor . get_cursors ( results , order_by ) { &quot;g3QAAAABdwRuYW1lbQAAAARNYXJ5&quot; , &quot;g3QAAAABdwRuYW1lbQAAAAVQZXRlcg==&quot; } iex&gt; iex&gt; Flop.Cursor . decode ( start_cursor ) { :ok , %{ name : &quot;Mary&quot; } } iex&gt; Flop.Cursor . decode ( end_cursor ) { :ok , %{ name : &quot;Peter&quot; } } If the result set is empty, the cursor values will be nil . iex&gt; Flop.Cursor . get_cursors ( [ ] , [ :id ] ) { nil , nil } The default function to retrieve the cursor value from the query result is Flop.Cursor.get_cursor_from_node/2 , which expects the query result to be a map or a 2-tuple. You can set the cursor_value_func option to use another function. Flop also comes with Flop.Cursor.get_cursor_from_edge/2 . If the records in the result set are not maps, you can define a custom cursor value function like this: iex&gt; results = [ { &quot;Mary&quot; , 1936 } , { &quot;Paul&quot; , 1937 } , { &quot;Peter&quot; , 1938 } ] iex&gt; cursor_func = fn { name , year } , order_fields -&gt; ...&gt; Enum . into ( order_fields , %{ } , fn ...&gt; :name -&gt; { :name , name } ...&gt; :year -&gt; { :year , year } ...&gt; end ) ...&gt; end iex&gt; opts = [ cursor_value_func : cursor_func ] iex&gt; iex&gt; { start_cursor , end_cursor } = ...&gt; Flop.Cursor . get_cursors ( results , [ :name , :year ] , opts ) { &quot;g3QAAAACdwRuYW1lbQAAAARNYXJ5dwR5ZWFyYgAAB5A=&quot; , &quot;g3QAAAACdwRuYW1lbQAAAAVQZXRlcncEeWVhcmIAAAeS&quot; } iex&gt; iex&gt; Flop.Cursor . decode ( start_cursor ) { :ok , %{ name : &quot;Mary&quot; , year : 1936 } } iex&gt; Flop.Cursor . decode ( end_cursor ) { :ok , %{ name : &quot;Peter&quot; , year : 1938 } }","ref":"Flop.Cursor.html#get_cursors/3"},{"type":"module","title":"Flop.Filter","doc":"Defines a filter.","ref":"Flop.Filter.html"},{"type":"function","title":"Flop.Filter.allowed_operators/1","doc":"Returns the allowed operators for the given Ecto type. If the given value is not a native Ecto type, a list with all operators is returned. iex&gt; allowed_operators ( :integer ) [ :== , :!= , :empty , :not_empty , :&lt;= , :&lt; , :&gt;= , :&gt; , :in , :not_in ]","ref":"Flop.Filter.html#allowed_operators/1"},{"type":"function","title":"Flop.Filter.allowed_operators/2","doc":"Returns the allowed operators for the given schema module and field. For regular Ecto schema fields, the type is derived via schema reflection. If the given schema module derives Flop.Schema , the type of join and custom fields is determined via the ecto_type option. Compound files are always handled as string fields, minus unsupported operators. If the type cannot be determined or if the type is not a native Ecto type, a list with all operators is returned. iex&gt; allowed_operators ( Pet , :age ) [ :== , :!= , :empty , :not_empty , :&lt;= , :&lt; , :&gt;= , :&gt; , :in , :not_in ]","ref":"Flop.Filter.html#allowed_operators/2"},{"type":"function","title":"Flop.Filter.delete/2","doc":"Deletes the filters for the given field from a list of filters. Examples Flop.Filter struct iex&gt; delete ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } ...&gt; ] , ...&gt; :name ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } ] iex&gt; delete ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } ] iex&gt; delete ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] Map with atom keys iex&gt; delete ( ...&gt; [ ...&gt; %{ field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; %{ field : :age , op : :&gt; , value : 8 } ...&gt; ] , ...&gt; :name ...&gt; ) [ %{ field : :age , op : :&gt; , value : 8 } ] Map with string keys iex&gt; delete ( ...&gt; [ ...&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } ] Indexed map Filters passed as an indexed map will be converted to a list, even if no matching filter exists. iex&gt; delete ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : &quot;8&quot; } ...&gt; } , ...&gt; :name ...&gt; ) [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : &quot;8&quot; } ] iex&gt; delete ( ...&gt; %{ ...&gt; &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; &quot;1&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } ...&gt; } , ...&gt; :name ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } ]","ref":"Flop.Filter.html#delete/2"},{"type":"function","title":"Flop.Filter.delete_first/2","doc":"Deletes the first filter in list of filters for the given field. Examples Flop.Filter struct iex&gt; delete_first ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ] iex&gt; delete_first ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] Map with atom keys iex&gt; delete_first ( ...&gt; [ ...&gt; %{ field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; %{ field : :age , op : :&gt; , value : 8 } , ...&gt; %{ field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ %{ field : :age , op : :&gt; , value : 8 } , %{ field : :name , op : :== , value : &quot;Jim&quot; } ] Map with string keys iex&gt; delete_first ( ...&gt; [ ...&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , ...&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ] iex&gt; delete_first ( ...&gt; [ ...&gt; %{ &quot;field&quot; =&gt; :name , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; %{ &quot;field&quot; =&gt; :age , &quot;op&quot; =&gt; :&gt; , &quot;value&quot; =&gt; 8 } , ...&gt; %{ &quot;field&quot; =&gt; :name , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ %{ &quot;field&quot; =&gt; :age , &quot;op&quot; =&gt; :&gt; , &quot;value&quot; =&gt; 8 } , %{ &quot;field&quot; =&gt; :name , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;Jim&quot; } ] Indexed map Filters passed as an indexed map will be converted to a list, even if no matching filter exists. iex&gt; delete_first ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 8 } , ...&gt; 2 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } ...&gt; } , ...&gt; :name ...&gt; ) [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 8 } , %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } ] iex&gt; delete_first ( ...&gt; %{ ...&gt; &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; &quot;1&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , ...&gt; &quot;2&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ...&gt; } , ...&gt; :name ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ]","ref":"Flop.Filter.html#delete_first/2"},{"type":"function","title":"Flop.Filter.drop/2","doc":"Removes the filters for the given fields from a list of filters. Examples Flop.Filter struct iex&gt; drop ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :color , op : :== , value : &quot;blue&quot; } ...&gt; ] , ...&gt; [ :name , :color ] ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } ] iex&gt; drop ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :color , op : :== , value : &quot;blue&quot; } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; [ :name , :species ] ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :color , op : :== , value : &quot;blue&quot; } ] Map with atom keys iex&gt; drop ( ...&gt; [ ...&gt; %{ field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; %{ field : :age , op : :&gt; , value : 8 } , ...&gt; %{ field : :color , op : :== , value : &quot;blue&quot; } ...&gt; ] , ...&gt; [ :name , :color ] ...&gt; ) [ %{ field : :age , op : :&gt; , value : 8 } ] Map with string keys iex&gt; drop ( ...&gt; [ ...&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } , ...&gt; %{ &quot;field&quot; =&gt; &quot;color&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;blue&quot; } ...&gt; ] , ...&gt; [ :name , :color ] ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } ] iex&gt; drop ( ...&gt; [ ...&gt; %{ &quot;field&quot; =&gt; :name , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; %{ &quot;field&quot; =&gt; :age , &quot;op&quot; =&gt; :&gt; , &quot;value&quot; =&gt; &quot;8&quot; } , ...&gt; %{ &quot;field&quot; =&gt; :color , &quot;op&quot; =&gt; :== , &quot;value&quot; =&gt; &quot;blue&quot; } ...&gt; ] , ...&gt; [ :name , :color ] ...&gt; ) [ %{ &quot;field&quot; =&gt; :age , &quot;op&quot; =&gt; :&gt; , &quot;value&quot; =&gt; &quot;8&quot; } ] Indexed map Filters passed as an indexed map will be converted to a list, even if no matching filter exists. iex&gt; drop ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : &quot;8&quot; } , ...&gt; 2 =&gt; %{ field : &quot;color&quot; , op : &quot;==&quot; , value : &quot;blue&quot; } ...&gt; } , ...&gt; [ :name , :color ] ...&gt; ) [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : &quot;8&quot; } ] iex&gt; drop ( ...&gt; %{ ...&gt; &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; &quot;1&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } , ...&gt; &quot;2&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;color&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;blue&quot; } ...&gt; } , ...&gt; [ :name , :color ] ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } ]","ref":"Flop.Filter.html#drop/2"},{"type":"function","title":"Flop.Filter.fetch/2","doc":"Fetches the first filter for the given field and returns it in a tuple. Examples Flop.Filter struct iex&gt; fetch ( [ ] , :name ) :error iex&gt; fetch ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { :ok , % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } } iex&gt; fetch ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) :error iex&gt; fetch ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) { :ok , % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } } Map with atom keys iex&gt; fetch ( [ %{ field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { :ok , %{ field : :name , op : :== , value : &quot;Joe&quot; } } Map with string keys iex&gt; fetch ( [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , :name ) { :ok , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } } Indexed map iex&gt; fetch ( ...&gt; %{ 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) { :ok , %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } } iex&gt; fetch ( ...&gt; %{ &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) { :ok , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } }","ref":"Flop.Filter.html#fetch/2"},{"type":"function","title":"Flop.Filter.fetch_value/2","doc":"Fetches the first filter value for the given field and returns it in a tuple. Examples Flop.Filter struct iex&gt; fetch_value ( [ ] , :name ) :error iex&gt; fetch_value ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { :ok , &quot;Joe&quot; } iex&gt; fetch_value ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) :error iex&gt; fetch_value ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) { :ok , &quot;Joe&quot; } Map with atom keys iex&gt; fetch_value ( [ %{ field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { :ok , &quot;Joe&quot; } Map with string keys iex&gt; fetch_value ( ...&gt; [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , ...&gt; :name ...&gt; ) { :ok , &quot;Joe&quot; } Indexed map iex&gt; fetch_value ( ...&gt; %{ 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) { :ok , &quot;Joe&quot; } iex&gt; fetch_value ( ...&gt; %{ &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) { :ok , &quot;Joe&quot; }","ref":"Flop.Filter.html#fetch_value/2"},{"type":"function","title":"Flop.Filter.get/2","doc":"Returns the first filter for the given field. Examples Flop.Filter struct iex&gt; get ( [ ] , :name ) nil iex&gt; get ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } iex&gt; get ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) nil iex&gt; get ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } Map with atom keys iex&gt; get ( [ %{ field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) %{ field : :name , op : :== , value : &quot;Joe&quot; } Map with string keys iex&gt; get ( [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , :name ) %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } iex&gt; get ( [ %{ &quot;field&quot; =&gt; :name , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , :name ) %{ &quot;field&quot; =&gt; :name , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } Indexed map iex&gt; get ( ...&gt; %{ 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } iex&gt; get ( ...&gt; %{ 0 =&gt; %{ field : :name , op : &quot;==&quot; , value : &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) %{ field : :name , op : &quot;==&quot; , value : &quot;Joe&quot; } iex&gt; get ( ...&gt; %{ &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } iex&gt; get ( ...&gt; %{ &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; :name , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) %{ &quot;field&quot; =&gt; :name , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; }","ref":"Flop.Filter.html#get/2"},{"type":"function","title":"Flop.Filter.get_all/2","doc":"Returns the all filters for the given field. Examples Flop.Filter struct iex&gt; get_all ( [ ] , :name ) [ ] iex&gt; get_all ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] iex&gt; get_all ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) [ ] iex&gt; get_all ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ] Map with atom keys iex&gt; get_all ( ...&gt; [ ...&gt; %{ field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; %{ field : :age , op : :&gt; , value : 8 } , ...&gt; %{ field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ %{ field : :name , op : :== , value : &quot;Joe&quot; } , %{ field : :name , op : :== , value : &quot;Jim&quot; } ] Map with string keys iex&gt; get_all ( ...&gt; [ ...&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , ...&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ] Indexed map iex&gt; get_all ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 8 } , ...&gt; 2 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } ...&gt; } , ...&gt; :name ...&gt; ) [ %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } ] iex&gt; get_all ( ...&gt; %{ ...&gt; &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; &quot;1&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , ...&gt; &quot;2&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ...&gt; } , ...&gt; :name ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ]","ref":"Flop.Filter.html#get_all/2"},{"type":"function","title":"Flop.Filter.get_value/2","doc":"Returns the first filter value for the given field. Examples Flop.Filter struct iex&gt; get_value ( [ ] , :name ) nil iex&gt; get_value ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) &quot;Joe&quot; iex&gt; get_value ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) nil iex&gt; get_value ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name ...&gt; ) &quot;Joe&quot; iex&gt; get_value ( [ % Flop.Filter { field : :ok , op : :empty , value : false } ] , :ok ) false Map with atom keys iex&gt; get_value ( [ %{ field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) &quot;Joe&quot; Map with string keys iex&gt; get_value ( ...&gt; [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , ...&gt; :name ...&gt; ) &quot;Joe&quot; Indexed map iex&gt; get_value ( ...&gt; %{ 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) &quot;Joe&quot; iex&gt; get_value ( ...&gt; %{ &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } } , ...&gt; :name ...&gt; ) &quot;Joe&quot;","ref":"Flop.Filter.html#get_value/2"},{"type":"function","title":"Flop.Filter.new/2","doc":"Creates a list of filters from an enumerable. The default operator is :== . iex&gt; filters = new ( %{ name : &quot;George&quot; , age : 8 } ) iex&gt; Enum . sort ( filters ) [ % Flop.Filter { field : :age , op : :== , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;George&quot; } ] iex&gt; filters = new ( [ name : &quot;George&quot; , age : 8 ] ) iex&gt; Enum . sort ( filters ) [ % Flop.Filter { field : :age , op : :== , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;George&quot; } ] You can optionally pass a mapping from field names to operators as a map with atom keys. iex&gt; filters = new ( ...&gt; %{ name : &quot;George&quot; , age : 8 } , ...&gt; operators : %{ name : :ilike_and } ...&gt; ) iex&gt; Enum . sort ( filters ) [ % Flop.Filter { field : :age , op : :== , value : 8 } , % Flop.Filter { field : :name , op : :ilike_and , value : &quot;George&quot; } ] You can also pass a map to rename fields. iex&gt; filters = new ( ...&gt; %{ s : &quot;George&quot; , age : 8 } , ...&gt; rename : %{ s : :name } ...&gt; ) iex&gt; Enum . sort ( filters ) [ % Flop.Filter { field : :age , op : :== , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;George&quot; } ] iex&gt; filters = new ( ...&gt; %{ s : &quot;George&quot; , cat : true } , ...&gt; rename : %{ s : :name , cat : :dog } ...&gt; ) iex&gt; Enum . sort ( filters ) [ % Flop.Filter { field : :dog , op : :== , value : true } , % Flop.Filter { field : :name , op : :== , value : &quot;George&quot; } ] If both a rename option and an operator are set for a field, the operator option needs to use the new field name. iex&gt; new ( ...&gt; %{ n : &quot;George&quot; } , ...&gt; rename : %{ n : :name } , ...&gt; operators : %{ name : :ilike_or } ...&gt; ) [ % Flop.Filter { field : :name , op : :ilike_or , value : &quot;George&quot; } ] If the enumerable uses string keys as field names, the function attempts to convert them to existing atoms. If the atom does not exist, the filter is removed from the list. iex&gt; new ( %{ &quot;name&quot; =&gt; &quot;George&quot; , &quot;age&quot; =&gt; 8 } ) [ % Flop.Filter { field : :age , op : :== , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;George&quot; } ] iex&gt; new ( %{ &quot;name&quot; =&gt; &quot;George&quot; , &quot;doesnotexist&quot; =&gt; 8 } ) [ % Flop.Filter { field : :name , op : :== , value : &quot;George&quot; } ] Other key types are also removed. iex&gt; new ( %{ &quot;name&quot; =&gt; &quot;George&quot; , 2 =&gt; 8 } ) [ % Flop.Filter { field : :name , op : :== , value : &quot;George&quot; } ]","ref":"Flop.Filter.html#new/2"},{"type":"function","title":"Flop.Filter.pop/3","doc":"Returns the first filter for the given field and removes all other filters for the same field from the filter list. Returns a tuple with the first matching filter for key and the remaining filter list. If there is no filter for the field in the list, the default value is returned as the first tuple element. See also Flop.Filter.pop_first/3 , Flop.Filter.pop_value/3 and Flop.Filter.pop_first_value/3 . Examples Flop.Filter struct iex&gt; pop ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , [ ] } iex&gt; pop ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) { nil , [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] } iex&gt; pop ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :age , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } ...&gt; ) { % Flop.Filter { field : :age , op : :&gt; , value : 8 } , [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] } iex&gt; pop ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } , ...&gt; ] , ...&gt; :name ...&gt; ) { % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } ] } Map with atom keys iex&gt; pop ( [ %{ field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { %{ field : :name , op : :== , value : &quot;Joe&quot; } , [ ] } Map with string keys iex&gt; pop ( [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , :name ) { %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , [ ] } Indexed map Filters passed as an indexed map will be converted to a list, even if no matching filter exists. iex&gt; pop ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : &quot;8&quot; } , ...&gt; 2 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } , ...&gt; } , ...&gt; :name ...&gt; ) { %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : &quot;8&quot; } ] } iex&gt; pop ( ...&gt; %{ ...&gt; 0 =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } , ...&gt; 2 =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } , ...&gt; } , ...&gt; :name ...&gt; ) { %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } ] }","ref":"Flop.Filter.html#pop/3"},{"type":"function","title":"Flop.Filter.pop_first/3","doc":"Returns the first filter for the given field and a filter list with all remaining filters. Returns a tuple with the first matching filter for key and the remaining filter list. If there is no filter for the field in the list, the default value is returned as the first tuple element. See also Flop.Filter.pop/3 , Flop.Filter.pop_value/3 and Flop.Filter.pop_first_value/3 . Examples Flop.Filter struct iex&gt; pop_first ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , [ ] } iex&gt; pop_first ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) { nil , [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] } iex&gt; pop_first ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :age , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } ...&gt; ) { % Flop.Filter { field : :age , op : :&gt; , value : 8 } , [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] } iex&gt; pop_first ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } , ...&gt; ] , ...&gt; :name ...&gt; ) { % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ] } Map with atom keys iex&gt; pop_first ( [ %{ field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { %{ field : :name , op : :== , value : &quot;Joe&quot; } , [ ] } Map with string keys iex&gt; pop_first ( ...&gt; [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , ...&gt; :name ...&gt; ) { %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , [ ] } Indexed map Filters passed as an indexed map will be converted to a list, even if no matching filter exists. iex&gt; pop_first ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 8 } , ...&gt; 2 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } , ...&gt; } , ...&gt; :name ...&gt; ) { %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 8 } , %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } ] } iex&gt; pop_first ( ...&gt; %{ ...&gt; &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; &quot;1&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , ...&gt; &quot;2&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } , ...&gt; } , ...&gt; :name ...&gt; ) { %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ] }","ref":"Flop.Filter.html#pop_first/3"},{"type":"function","title":"Flop.Filter.pop_first_value/3","doc":"Returns the first filter for the given field and a filter list with all remaining filters. Returns a tuple with the first matching filter value for key and the remaining filter list. If there is no filter for the field in the list, the default value is returned as the first tuple element. See also Flop.Filter.pop/3 , Flop.Filter.pop_value/3 and Flop.Filter.pop_first/3 . Examples Flop.Filter struct iex&gt; pop_first_value ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :name ...&gt; ) { &quot;Joe&quot; , [ ] } iex&gt; pop_first_value ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :age ...&gt; ) { nil , [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] } iex&gt; pop_first_value ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :age , ...&gt; 8 ...&gt; ) { 8 , [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] } iex&gt; pop_first_value ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } , ...&gt; ] , ...&gt; :name ...&gt; ) { &quot;Joe&quot; , [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ] } Map with atom keys iex&gt; pop_first_value ( [ %{ field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { &quot;Joe&quot; , [ ] } Map with string keys iex&gt; pop_first_value ( ...&gt; [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , ...&gt; :name ...&gt; ) { &quot;Joe&quot; , [ ] } Indexed map Filters passed as an indexed map will be converted to a list, even if no matching filter exists. iex&gt; pop_first_value ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 8 } , ...&gt; 2 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } , ...&gt; } , ...&gt; :name ...&gt; ) { &quot;Joe&quot; , [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 8 } , %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } ] } iex&gt; pop_first_value ( ...&gt; %{ ...&gt; &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; &quot;1&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , ...&gt; &quot;2&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } , ...&gt; } , ...&gt; :name ...&gt; ) { &quot;Joe&quot; , [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } ] }","ref":"Flop.Filter.html#pop_first_value/3"},{"type":"function","title":"Flop.Filter.pop_value/3","doc":"Returns the first filter value for the given field and removes all other filters for the same field from the filter list. Returns a tuple with the value of the first matching filter for key and the remaining filter list. If there is no filter for the field in the list, the default value is returned as the first tuple element. See also Flop.Filter.pop/3 , Flop.Filter.pop_first/3 and Flop.Filter.pop_first_value/3 Examples Flop.Filter struct iex&gt; pop_value ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { &quot;Joe&quot; , [ ] } iex&gt; pop_value ( [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , :age ) { nil , [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] } iex&gt; pop_value ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :age , ...&gt; 8 ...&gt; ) { 8 , [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] } iex&gt; pop_value ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } , ...&gt; ] , ...&gt; :name ...&gt; ) { &quot;Joe&quot; , [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } ] } Map with atom keys iex&gt; pop_value ( [ %{ field : :name , op : :== , value : &quot;Joe&quot; } ] , :name ) { &quot;Joe&quot; , [ ] } Map with string keys iex&gt; pop_value ( [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } ] , :name ) { &quot;Joe&quot; , [ ] } Indexed map Filters passed as an indexed map will be converted to a list, even if no matching filter exists. iex&gt; pop_value ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : &quot;8&quot; } , ...&gt; 2 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Jim&quot; } , ...&gt; } , ...&gt; :name ...&gt; ) { &quot;Joe&quot; , [ %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : &quot;8&quot; } ] } iex&gt; pop_value ( ...&gt; %{ ...&gt; 0 =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } , ...&gt; 2 =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Jim&quot; } , ...&gt; } , ...&gt; :name ...&gt; ) { &quot;Joe&quot; , [ %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; &quot;8&quot; } ] }","ref":"Flop.Filter.html#pop_value/3"},{"type":"function","title":"Flop.Filter.put/2","doc":"Adds the given filter to the filter list and removes all existing filters for the same field from the list. Examples iex&gt; put ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] iex&gt; put ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jane&quot; } ...&gt; ) [ % Flop.Filter { field : :name , op : :== , value : &quot;Jane&quot; } , % Flop.Filter { field : :age , op : :&gt; , value : 8 } ]","ref":"Flop.Filter.html#put/2"},{"type":"function","title":"Flop.Filter.put_new/2","doc":"Adds the given filter to the filter list only if no filter for the field exists yet. Examples iex&gt; put_new ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] iex&gt; put_new ( ...&gt; [ ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jane&quot; } ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ]","ref":"Flop.Filter.html#put_new/2"},{"type":"function","title":"Flop.Filter.put_new_value/4","doc":"Adds the given filter value to the filter list only if no filter for the field exists yet. Examples iex&gt; put_new_value ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :age , ...&gt; 8 , ...&gt; :&gt; ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] iex&gt; put_new_value ( ...&gt; [ ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name , ...&gt; &quot;Jane&quot; ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt; , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ]","ref":"Flop.Filter.html#put_new_value/4"},{"type":"function","title":"Flop.Filter.put_value/4","doc":"Adds the given filter value to the filter list and removes all existing filters for the same field from the list. Examples iex&gt; put_value ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :age , ...&gt; 8 ...&gt; ) [ % Flop.Filter { field : :age , op : :== , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] iex&gt; put_value ( ...&gt; [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] , ...&gt; :age , ...&gt; 8 , ...&gt; :&gt;= ...&gt; ) [ % Flop.Filter { field : :age , op : :&gt;= , value : 8 } , % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } ] iex&gt; put_value ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; :name , ...&gt; &quot;Jane&quot; ...&gt; ) [ % Flop.Filter { field : :name , op : :== , value : &quot;Jane&quot; } , % Flop.Filter { field : :age , op : :&gt; , value : 8 } ]","ref":"Flop.Filter.html#put_value/4"},{"type":"function","title":"Flop.Filter.take/2","doc":"Takes all filters for the given fields from a filter list. Examples Flop.Filter struct iex&gt; take ( ...&gt; [ ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; % Flop.Filter { field : :age , op : :&gt; , value : 8 } , ...&gt; % Flop.Filter { field : :color , op : :== , value : &quot;blue&quot; } , ...&gt; % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ...&gt; ] , ...&gt; [ :name , :color ] ...&gt; ) [ % Flop.Filter { field : :name , op : :== , value : &quot;Joe&quot; } , % Flop.Filter { field : :color , op : :== , value : &quot;blue&quot; } , % Flop.Filter { field : :name , op : :== , value : &quot;Jim&quot; } ] Map with atom keys iex&gt; take ( ...&gt; [ ...&gt; %{ field : :name , op : :== , value : &quot;Joe&quot; } , ...&gt; %{ field : :age , op : :&gt; , value : 8 } , ...&gt; %{ field : :color , op : :== , value : &quot;blue&quot; } ...&gt; ] , ...&gt; [ :name , :color ] ...&gt; ) [ %{ field : :name , op : :== , value : &quot;Joe&quot; } , %{ field : :color , op : :== , value : &quot;blue&quot; } ] Map with string keys iex&gt; take ( ...&gt; [ ...&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , ...&gt; %{ &quot;field&quot; =&gt; &quot;color&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;blue&quot; } ...&gt; ] , ...&gt; [ :name , :color ] ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , %{ &quot;field&quot; =&gt; &quot;color&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;blue&quot; } ] Indexed map Filters passed as an indexed map will be converted to a list, even if no matching filter exists. iex&gt; take ( ...&gt; %{ ...&gt; 0 =&gt; %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , ...&gt; 1 =&gt; %{ field : &quot;age&quot; , op : &quot;&gt;&quot; , value : 8 } , ...&gt; 2 =&gt; %{ field : &quot;color&quot; , op : &quot;==&quot; , value : &quot;blue&quot; } ...&gt; } , ...&gt; [ :name , :color ] ...&gt; ) [ %{ field : &quot;name&quot; , op : &quot;==&quot; , value : &quot;Joe&quot; } , %{ field : &quot;color&quot; , op : &quot;==&quot; , value : &quot;blue&quot; } ] iex&gt; take ( ...&gt; %{ ...&gt; &quot;0&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , ...&gt; &quot;1&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;age&quot; , &quot;op&quot; =&gt; &quot;&gt;&quot; , &quot;value&quot; =&gt; 8 } , ...&gt; &quot;2&quot; =&gt; %{ &quot;field&quot; =&gt; &quot;color&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;blue&quot; } ...&gt; } , ...&gt; [ :name , :color ] ...&gt; ) [ %{ &quot;field&quot; =&gt; &quot;name&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;Joe&quot; } , %{ &quot;field&quot; =&gt; &quot;color&quot; , &quot;op&quot; =&gt; &quot;==&quot; , &quot;value&quot; =&gt; &quot;blue&quot; } ]","ref":"Flop.Filter.html#take/2"},{"type":"type","title":"Flop.Filter.op/0","doc":"Represents valid filter operators. Operator Value WHERE clause :== &quot;Salicaceae&quot; WHERE column = 'Salicaceae' :!= &quot;Salicaceae&quot; WHERE column != 'Salicaceae' :=~ &quot;cyth&quot; WHERE column ILIKE '%cyth%' :empty true WHERE (column IS NULL) = true :empty false WHERE (column IS NULL) = false :not_empty true WHERE (column IS NOT NULL) = true :not_empty false WHERE (column IS NOT NULL) = false :&lt;= 10 WHERE column &lt;= 10 :&lt; 10 WHERE column &lt; 10 :&gt;= 10 WHERE column &gt;= 10 :&gt; 10 WHERE column &gt; 10 :in [&quot;pear&quot;, &quot;plum&quot;] WHERE column = ANY('pear', 'plum') :not_in [&quot;pear&quot;, &quot;plum&quot;] WHERE column = NOT IN('pear', 'plum') :contains &quot;pear&quot; WHERE 'pear' = ANY(column) :not_contains &quot;pear&quot; WHERE 'pear' = NOT IN(column) :like &quot;cyth&quot; WHERE column LIKE '%cyth%' :not_like &quot;cyth&quot; WHERE column NOT LIKE '%cyth%' :like_and [&quot;Rubi&quot;, &quot;Rosa&quot;] WHERE column LIKE '%Rubi%' AND column LIKE '%Rosa%' :like_and &quot;Rubi Rosa&quot; WHERE column LIKE '%Rubi%' AND column LIKE '%Rosa%' :like_or [&quot;Rubi&quot;, &quot;Rosa&quot;] WHERE column LIKE '%Rubi%' OR column LIKE '%Rosa%' :like_or &quot;Rubi Rosa&quot; WHERE column LIKE '%Rubi%' OR column LIKE '%Rosa%' :ilike &quot;cyth&quot; WHERE column ILIKE '%cyth%' :not_ilike &quot;cyth&quot; WHERE column NOT ILIKE '%cyth%' :ilike_and [&quot;Rubi&quot;, &quot;Rosa&quot;] WHERE column ILIKE '%Rubi%' AND column ILIKE '%Rosa%' :ilike_and &quot;Rubi Rosa&quot; WHERE column ILIKE '%Rubi%' AND column ILIKE '%Rosa%' :ilike_or [&quot;Rubi&quot;, &quot;Rosa&quot;] WHERE column ILIKE '%Rubi%' OR column ILIKE '%Rosa%' :ilike_or &quot;Rubi Rosa&quot; WHERE column ILIKE '%Rubi%' OR column ILIKE '%Rosa%' The filter operators :empty and :not_empty will regard empty arrays as empty values if the field is known to be an array field. The filter operators :ilike_and , :ilike_or , :like_and and :like_or accept both strings and list of strings. If the filter value is a string, it will be split at whitespace characters and the segments are combined with and or or . If a list of strings is passed, the individual strings are not split, and the list items are combined with and or or .","ref":"Flop.Filter.html#t:op/0"},{"type":"type","title":"Flop.Filter.t/0","doc":"Represents filter query parameters. Fields field : The field the filter is applied to. The allowed fields can be restricted by deriving Flop.Schema in your Ecto schema. op : The filter operator. value : The comparison value of the filter.","ref":"Flop.Filter.html#t:t/0"},{"type":"module","title":"Flop.Meta","doc":"Defines a struct for holding meta information of a query result.","ref":"Flop.Meta.html"},{"type":"function","title":"Flop.Meta.with_errors/3","doc":"Returns a Flop.Meta struct with the given params, errors, and opts. This function is used internally to build error responses in case of validation errors. You can use it to add additional parameter validation. The given parameters parameters are normalized before being added to the struct. The errors have to be passed as a keyword list (same format as the result of Ecto.Changeset.traverse_errors(changeset, &amp; &amp;1) ). Example In this list function, the given parameters are first validated with Flop.validate/2 , which returns a Flop struct on success. You can then pass that struct to a custom validation function, along with the original parameters and the opts, which both are needed to call this function. def list_pets ( %{ } = params ) do opts = [ for : Pet ] with { :ok , % Flop { } = flop } &lt;- Flop . validate ( params , opts ) , { :ok , % Flop { } = flop } &lt;- custom_validation ( flop , params , opts ) do Flop . run ( Pet , flop , for : Pet ) end end In your custom validation function, you can retrieve and manipulate the filter values in the Flop struct with the functions defined in the Flop.Filter module. defp custom_validation ( % Flop { } = flop , %{ } = params , opts ) do %{ value : date } = Flop.Filter . get ( flop . filters , :date ) if date &amp;&amp; Date . compare ( date , Date . utc_today ( ) ) != :lt do errors = [ filters : [ { &quot;date must be in the past&quot; , [ ] } ] ] { :error , Flop.Meta . with_errors ( params , errors , opts ) } else { :ok , flop } end end Note that in this example, Flop.Filter.get/2 is used, which only returns the first filter in the given filter list. Depending on how you use Flop, the filter list may have multiple entries for the same field. In that case, you may need to either use Flop.Filter.get_all/2 and apply the validation on all returned filters, or reduce over the whole filter list. The latter has the advantage that you can attach the error to the actual list entry. def custom_validation ( % Flop { } = flop , %{ } = params , opts ) do filter_errors = flop . filters |&gt; Enum . reduce ( [ ] , &amp; validate_filter / 2 ) |&gt; Enum . reverse ( ) if Enum . any? ( filter_errors , &amp; ( &amp;1 != [ ] ) ) do errors = [ filters : filter_errors ] { :error , Flop.Meta . with_errors ( params , errors , opts ) } else { :ok , flop } end end defp validate_filter ( % Flop.Filter { field : :date , value : date } , acc ) when is_binary ( date ) do date = Date . from_iso8601! ( date ) if Date . compare ( date , Date . utc_today ( ) ) != :lt , do : [ [ value : [ { &quot;date must be in the past&quot; , [ ] } ] ] | acc ] , else : [ [ ] | acc ] end defp validate_filter ( % Flop.Filter { } , acc ) , do : [ [ ] | acc ]","ref":"Flop.Meta.html#with_errors/3"},{"type":"type","title":"Flop.Meta.t/0","doc":"Meta information for a query result. :flop - The Flop struct used in the query. :schema - The schema module passed as for option. :backend - The backend module if the query was made using a module with use Flop . :current_offset - The :offset value used in the query when using offset-based pagination or a derived value when using page-based pagination. Always nil when using cursor-based pagination. :current_page - The :page value used in the query when using page-based pagination or a derived value when using offset-based pagination. Note that the value will be rounded if the offset lies between pages. Always nil when using cursor-based pagination. :errors - Any validation errors that occurred. The format is the same as the result of Ecto.Changeset.traverse_errors(changeset, &amp; &amp;1) . :previous_offset , :next_offset , :previous_page , :next_page - Values based on :current_page and :current_offset / page_size . Always nil when using cursor-based pagination. :start_cursor , :end_cursor - The cursors of the first and last record in the result set. Only set when using cursor-based pagination with :first / :after or :last / :before . :has_previous_page? , :has_next_page? - Set in all pagination types. Note that :has_previous_page? is always true when using cursor-based pagination with :first and :after is set; likewise, :has_next_page? is always true when using cursor-based pagination with :before and :last is set. :page_size - The page size or limit of the query. Set to the :first or :last parameter when using cursor-based pagination. :params - The original, unvalidated params that were passed. Only set if validation errors occurred. :total_count - The total count of records for the given query. Always nil when using cursor-based pagination. :total_pages - The total page count based on the total record count and the page size. Always nil when using cursor-based pagination.","ref":"Flop.Meta.html#t:t/0"},{"type":"module","title":"Flop.Relay","doc":"Helpers to turn query results into Relay formats.","ref":"Flop.Relay.html"},{"type":"function","title":"Flop.Relay.connection_from_result/2","doc":"Takes the query results returned by Flop.run/3 , Flop.validate_and_run/3 or Flop.validate_and_run!/3 and turns them into the Relay connection format. Example iex&gt; flop = % Flop { order_by : [ :name ] } iex&gt; meta = % Flop.Meta { flop : flop , start_cursor : &quot;a&quot; , end_cursor : &quot;b&quot; } iex&gt; result = { [ % MyApp.Fruit { name : &quot;Apple&quot; , family : &quot;Rosaceae&quot; } ] , meta } iex&gt; Flop.Relay . connection_from_result ( result ) %{ edges : [ %{ cursor : &quot;g3QAAAABdwRuYW1lbQAAAAVBcHBsZQ==&quot; , node : % MyApp.Fruit { family : &quot;Rosaceae&quot; , id : nil , name : &quot;Apple&quot; } } ] , page_info : %{ end_cursor : &quot;b&quot; , has_next_page : false , has_previous_page : false , start_cursor : &quot;a&quot; } } See Flop.Relay.edges_from_result/2 for an example of adding additional fields to the edge. Options :cursor_value_func : 2-arity function that takes an item from the query result and the order_by fields and returns the unencoded cursor value.","ref":"Flop.Relay.html#connection_from_result/2"},{"type":"function","title":"Flop.Relay.edges_from_result/2","doc":"Turns a list of query results into Relay edges. Simple queries If your query returns a list of maps or structs, the function will return the a list of edges with :cursor and :node as only fields. iex&gt; flop = % Flop { order_by : [ :name ] } iex&gt; meta = % Flop.Meta { flop : flop } iex&gt; result = { [ % MyApp.Fruit { name : &quot;Apple&quot; , family : &quot;Rosaceae&quot; } ] , meta } iex&gt; Flop.Relay . edges_from_result ( result ) [ %{ cursor : &quot;g3QAAAABdwRuYW1lbQAAAAVBcHBsZQ==&quot; , node : % MyApp.Fruit { name : &quot;Apple&quot; , family : &quot;Rosaceae&quot; } } ] Supplying additional edge information If the query result is a list of 2-tuples, this is interpreted as a tuple of the node information and the edge information. For example, if you have a query like this: Group |&gt; where ( [ g ] , g . id == ^ group_id ) |&gt; join ( :left , [ g ] , m in assoc ( g , :members ) ) |&gt; select ( [ g , m ] , { m , map ( m , [ :role ] ) } ) Then your query result looks something like: [ { % Member { id : 242 , name : &quot;Carl&quot; } , %{ role : :owner } } ] In this case, the members are the nodes, and the maps with the roles is seen as edge information. [ %{ cursor : &quot;AE98RNSTNGN&quot; , node : % Member { id : 242 , name : &quot;Carl&quot; } , role : :owner } ] Note that in this case, the whole tuple will be passed to the cursor value function, so that the cursor can be based on both node and edge fields. Here's an example with fruit that overrides the cursor value function: iex&gt; flop = % Flop { order_by : [ :name ] } iex&gt; meta = % Flop.Meta { flop : flop } iex&gt; items = [ { % MyApp.Fruit { name : &quot;Apple&quot; } , %{ preparation : :grated } } ] iex&gt; func = fn { fruit , _edge } , order_by -&gt; Map . take ( fruit , order_by ) end iex&gt; Flop.Relay . edges_from_result ( ...&gt; { items , meta } , ...&gt; cursor_value_func : func ...&gt; ) [ %{ cursor : &quot;g3QAAAABdwRuYW1lbQAAAAVBcHBsZQ==&quot; , node : % MyApp.Fruit { name : &quot;Apple&quot; } , preparation : :grated } ] Options :cursor_value_func : 2-arity function that takes an item from the query result and the order_by fields and returns the unencoded cursor value.","ref":"Flop.Relay.html#edges_from_result/2"},{"type":"function","title":"Flop.Relay.page_info_from_meta/1","doc":"Takes a Flop.Meta struct and returns a map with the Relay page info. Example iex&gt; Flop.Relay . page_info_from_meta ( % Flop.Meta { ...&gt; has_previous_page? : true , ...&gt; has_next_page? : true , ...&gt; start_cursor : &quot;a&quot; , ...&gt; end_cursor : &quot;b&quot; ...&gt; } ) %{ has_previous_page : true , has_next_page : true , start_cursor : &quot;a&quot; , end_cursor : &quot;b&quot; }","ref":"Flop.Relay.html#page_info_from_meta/1"},{"type":"type","title":"Flop.Relay.connection/0","doc":"","ref":"Flop.Relay.html#t:connection/0"},{"type":"type","title":"Flop.Relay.edge/0","doc":"","ref":"Flop.Relay.html#t:edge/0"},{"type":"type","title":"Flop.Relay.page_info/0","doc":"","ref":"Flop.Relay.html#t:page_info/0"},{"type":"protocol","title":"Flop.Schema","doc":"Flop.Schema is a protocol that allows you to customize and set query options in your Ecto schemas. This module allows you to define which fields are filterable and sortable, set default and maximum limits, specify default sort orders, restrict pagination types, and more. Usage To utilize this protocol, derive Flop.Schema in your Ecto schema and define the filterable and sortable fields. defmodule MyApp.Pet do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] } schema &quot;pets&quot; do field :name , :string field :age , :integer field :species , :string end end See option/0 for an overview of all available options. @derive Flop.Schema When you derive Flop.Schema , all the functions required for the Flop.Schema protocol will be defined based on the options you set. After that, you can pass the module as the :for option to Flop.validate/2 . iex&gt; Flop . validate ( % Flop { order_by : [ :name ] } , for : MyApp.Pet ) { :ok , % Flop { filters : [ ] , limit : 50 , offset : nil , order_by : [ :name ] , order_directions : nil , page : nil , page_size : nil } } iex&gt; { :error , % Flop.Meta { } = meta } = Flop . validate ( ...&gt; % Flop { order_by : [ :species ] } , for : MyApp.Pet ...&gt; ) iex&gt; meta . params %{ &quot;order_by&quot; =&gt; [ :species ] , &quot;filters&quot; =&gt; [ ] } iex&gt; meta . errors [ order_by : [ { &quot;has an invalid entry&quot; , [ validation : :subset , enum : [ :name , :age , :owner_name , :owner_age ] ] } ] ] Default and maximum limits Define a default or maximum limit by setting the default_limit and max_limit options while deriving Flop.Schema. Flop.validate/1 will apply the default limit and validate the maximum limit. @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , max_limit : 100 , default_limit : 50 } Default sort order Specify a default sort order by setting the default_order_by and default_order_directions options when deriving Flop.Schema. The default values will be applied by Flop.validate/1 . If no order directions are set, :asc is the default for all fields. @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , default_order : %{ order_by : [ :name , :age ] , order_directions : [ :asc , :desc ] } } Restricting pagination types By default, page / page_size , offset / limit and cursor-based pagination ( first / after and last / before ) are enabled. If you wish to restrict the pagination type for a schema, you can set the pagination_types option. @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , pagination_types : [ :first , :last ] } See also Flop.option/0 and Flop.pagination_type/0 . Setting the value to nil allows all pagination types. Alias fields To sort by calculated values, you can use Ecto.Query.API.selected_as/2 in your query, define an alias field in your schema, and add the alias field to the list of sortable fields. Schema: @derive { Flop.Schema , filterable : [ ] , sortable : [ :pet_count ] , alias_fields : [ :pet_count ] } Query: Owner |&gt; join ( :left , [ o ] , p in assoc ( o , :pets ) , as : :pets ) |&gt; group_by ( [ o ] , o . id ) |&gt; select ( [ o , pets : p ] , { o . id , p . id |&gt; count ( ) |&gt; selected_as ( :pet_count ) } ) |&gt; Flop . validate_and_run ( params , for : Owner ) Note that it is not possible to use field aliases in WHERE clauses, which means you cannot add alias fields to the list of filterable fields, and you cannot sort by an alias field if you are using cursor-based pagination. Compound fields Sometimes you might need to apply a search term to multiple fields at once, e.g. you might want to search in both the family name and given name field. You can do that with Flop by defining a compound field. @derive { Flop.Schema , filterable : [ :full_name ] , sortable : [ :full_name ] , compound_fields : [ full_name : [ :family_name , :given_name ] ] } This allows you to use the field name :full_name as any other field in the filter and order parameters. Filtering params = %{ filters : [ %{ field : :full_name , op : :like , value : &quot;margo&quot; } ] } This would translate to: WHERE family_name like &#39;%margo%&#39; OR given_name like &#39;%margo%&#39; Partial matches of the search term can be achieved with one of the like operators. params = %{ filters : [ %{ field : :full_name , op : :ilike_and , value : [ &quot;margo&quot; , &quot;martindale&quot; ] } ] } or params = %{ filters : [ %{ field : :full_name , op : :ilike_and , value : &quot;margo martindale&quot; } ] } This would translate to: WHERE ( family_name ilike &#39;%margo%&#39; OR given_name ilike &#39;%margo%&#39; ) AND ( family_name ilike &#39;%martindale%&#39; OR given_name ilike &#39;%martindale%&#39; ) Filter operator rules :=~ :like :not_like :like_and :like_or :ilike :not_ilike :ilike_and :ilike_or If a string value is passed it will be split at whitespace characters and each segment will be checked separately. If a list of strings is passed the individual strings are not split. The filter matches for a value if it matches for any of the fields. :empty Matches if all fields of the compound field are nil . :not_empty Matches if any field of the compound field is not nil . :== :!= :&lt;= :&lt; :&gt;= :&gt; :in :not_in :contains :not_contains These filter operators are ignored for compound fields at the moment. This will be added in a future version. The filter value is normalized by splitting the string at whitespaces and joining it with a space. The values of all fields of the compound field are split by whitespace character and joined with a space, and the resulting values are joined with a space again. Sorting params = %{ order_by : [ :full_name ] , order_directions : [ :desc ] } This would translate to: ORDER BY family_name DESC , given_name DESC Note that compound fields cannot be used as pagination cursors. Join fields If you need to filter or order across tables, you can define join fields. As an example, let's define these schemas: schema &quot;owners&quot; do field :name , :string field :email , :string has_many :pets , Pet end schema &quot;pets&quot; do field :name , :string field :species , :string belongs_to :owner , Owner end And now we want to find all owners that have pets of the species &quot;E. africanus&quot; . To do this, first we need to define a join field on the Owner schema. @derive { Flop.Schema , filterable : [ :pet_species ] , sortable : [ :pet_species ] , join_fields : [ pet_species : [ binding : :pets , field : :species , ecto_type : :string ] ] } In this case, :pet_species would be the alias of the field that you can refer to in the filter and order parameters. The options are: :binding - The named binding you set with the :as option in the join statement of your query. :field - The field on that binding on which the filter should be applied. :ecto_type - The Ecto type of the field. This allows Flop to validate filter values, and also to treat empty arrays and empty maps as empty values depending on the type. See also Ecto type option section below. In order to retrieve the pagination cursor value for a join field, Flop needs to know how to get the field value from the struct that is returned from the database. Flop.Schema.get_field/2 is used for that. By default, Flop assumes that the binding name matches the name of the field for the association in your Ecto schema (the one you set with has_one , has_many or belongs_to ). In the example above, Flop would try to access the field in the struct under the path [:pets, :species] . If you have joins across multiple tables, or if you can't give the binding the same name as the association field, you can specify the path explicitly. @derive { Flop.Schema , filterable : [ :pet_species ] , sortable : [ :pet_species ] , join_fields : [ pet_species : [ binding : :pets , field : :species , path : [ :pets , :species ] ] } After setting up the join fields, you can write a query like this: params = %{ filters : [ %{ field : :pet_species , op : :== , value : &quot;E. africanus&quot; } ] } Owner |&gt; join ( :left , [ o ] , p in assoc ( o , :pets ) , as : :pets ) |&gt; preload ( [ pets : p ] , [ pets : p ] ) |&gt; Flop . validate_and_run! ( params , for : Owner ) If your query returns data in a different format, you don't need to set the :path option. Instead, you can pass a custom cursor value function in the options. See Flop.Cursor.get_cursors/2 and Flop.option/0 . Note that Flop doesn't create the join clauses for you. The named bindings already have to be present in the query you pass to the Flop functions. You can use Flop.with_named_bindings/4 or Flop.named_bindings/3 to get the build the join clauses needed for a query dynamically and avoid adding unnecessary joins. Filtering by calculated values with subqueries You can join on a subquery with a named binding and add a join field as described above. Schema: @derive { Flop.Schema , filterable : [ :pet_count ] , sortable : [ :pet_count ] , join_fields : [ pet_count : [ { :pet_count , :count } ] } Query: params = %{ filters : [ %{ field : :pet_count , op : :&gt; , value : 2 } ] } pet_count_query = Pet |&gt; where ( [ p ] , parent_as ( :owner ) . id == p . owner_id ) |&gt; select ( [ p ] , %{ count : count ( p ) } ) q = ( o in Owner ) |&gt; from ( as : :owner ) |&gt; join ( :inner_lateral , [ owner : o ] , p in subquery ( pet_count_query ) , as : :pet_count ) |&gt; Flop . validate_and_run ( params , for : Owner ) Custom fields Custom fields allow for precise control over filter queries, making it possible to implement filter logic that the built-in filtering options cannot satisfy. For example, you might need to handle dates and times in a particular way that takes into account different time zones, or perform database-specific queries using fragments. Custom field filters are referenced by a tuple {mod :: module, function :: atom, opts :: keyword} . The referenced function receives three arguments: the Ecto query, the Flop filter, and an options keyword list. If runtime options are necessary (like the timezone of the request or the user ID of the current user), use the extra_opts option when calling Flop functions. Note that as of now, custom fields only support filtering, not sorting. Schema: @derive { Flop.Schema , filterable : [ :inserted_at_date ] , custom_fields : [ inserted_at_date : [ filter : { CustomFilters , :date_filter , [ source : :inserted_at ] } , ecto_type : :date , operators : [ :&lt;= , :&gt;= ] ] ] } If you pass the :ecto_type option like above, the filter value will be automatically cast. Filter module: defmodule CustomFilters do import Ecto.Query def date_filter ( query , % Flop.Filter { value : value , op : op } , opts ) do source = Keyword . fetch! ( opts , :source ) timezone = Keyword . fetch! ( opts , :timezone ) expr = dynamic ( [ r ] , fragment ( &quot;((? AT TIME ZONE &#39;utc&#39;) AT TIME ZONE ?)::date&quot; , field ( r , ^ source ) , ^ timezone ) ) conditions = case op do :&gt;= -&gt; dynamic ( [ r ] , ^ expr &gt;= ^ date ) :&lt;= -&gt; dynamic ( [ r ] , ^ expr &lt;= ^ date ) end where ( query , ^ conditions ) end end Query: Flop . validate_and_run ( MyApp.Pet , params , for : MyApp.Pet , extra_opts : [ timezone : timezone ] ) If your custom filter requires certain named bindings, you can use the :bindings option to specify them. Then, using Flop.with_named_bindings/4 , these bindings can be conditionally added to your query based on filter conditions. Ecto type option Flop automatically retrieves the field type from the schema module for regular schema fields, enabling it to correctly cast filter values. Compound fields are always treated as string fields. For join and custom fields, Flop cannot automatically determine the Ecto type. Therefore, you need to specify the ecto_type option. This helps Flop cast filter values for join and custom fields properly. If this option is not set, Flop will accept any filter value, potentially leading to an Ecto.Query.CastError if an invalid filter value is used. Additionally, without this option, Flop cannot identify empty lists and maps as empty values for array and map fields. @derive { Flop.Schema , filterable : [ :full_text , :pet_species ] , sortable : [ :id ] , join_fields : [ pet_species : [ binding : :pets , field : :species , ecto_type : :string ] , full_text : [ filter : { __MODULE__ , :full_text_filter , [ ] } , ecto_type : :string ] ] } You can specify any Ecto type with the ecto_type option. Here are some examples: A simple string: ecto_type: :string An integer: ecto_type: :integer An array of strings: ecto_type: {:array, :string} A custom Ecto type: ecto_type: MyCustomType For parameterized types, use the following syntax: ecto_type: {:parameterized, Ecto.Enum, Ecto.Enum.init(values: [:one, :two])} If you're working with Ecto.Enum types, you can use a more convenient syntax: ecto_type: {:ecto_enum, [:one, :two]} Furthermore, you can reference a type from another schema: ecto_type: {:from_schema, MyApp.Pet, :mood} Note that Flop.Phoenix encodes all filters in query string using Plug.Conn.Query . It is expected that filter values can be converted to strings with to_string/1 . If you are using an Ecto custom type that casts as a struct, you will need to implement the String.Chars protocol for that struct.","ref":"Flop.Schema.html"},{"type":"function","title":"Flop.Schema.default_limit/1","doc":"Returns the default limit of a schema. iex&gt; Flop.Schema . default_limit ( % MyApp.Fruit { } ) 60","ref":"Flop.Schema.html#default_limit/1"},{"type":"function","title":"Flop.Schema.default_order/1","doc":"Returns the default order of a schema. iex&gt; Flop.Schema . default_order ( % MyApp.Fruit { } ) %{ order_by : [ :name ] , order_directions : [ :asc ] }","ref":"Flop.Schema.html#default_order/1"},{"type":"function","title":"Flop.Schema.default_pagination_type/1","doc":"Returns the allowed pagination types of a schema. iex&gt; Flop.Schema . pagination_types ( % MyApp.Fruit { } ) [ :first , :last , :offset ]","ref":"Flop.Schema.html#default_pagination_type/1"},{"type":"function","title":"Flop.Schema.field_type/2","doc":"Returns the field type in a schema. {:normal, atom} - An ordinary field on the schema. The second tuple element is the field name. {:compound, [atom]} - A combination of fields defined with the compound_fields option. The list of atoms refers to the list of fields that are included. {:join, map} - A field from a named binding as defined with the join_fields option. The map has keys for the :binding , :field and :path . {:custom, keyword} - A filter field that uses a custom filter function. Examples iex&gt; field_type ( % MyApp.Pet { } , :age ) { :normal , :age } iex&gt; field_type ( % MyApp.Pet { } , :full_name ) { :compound , [ :family_name , :given_name ] } iex&gt; field_type ( % MyApp.Pet { } , :owner_name ) { :join , %{ binding : :owner , field : :name , path : [ :owner , :name ] , ecto_type : :string } } iex&gt; field_type ( % MyApp.Pet { } , :reverse_name ) { :custom , %{ filter : { MyApp.Pet , :reverse_name_filter , [ ] } , ecto_type : :string , operators : nil , bindings : [ ] } }","ref":"Flop.Schema.html#field_type/2"},{"type":"function","title":"Flop.Schema.filterable/1","doc":"Returns the filterable fields of a schema. iex&gt; Flop.Schema . filterable ( % MyApp.Pet { } ) [ :age , :full_name , :mood , :name , :owner_age , :owner_name , :owner_tags , :pet_and_owner_name , :species , :tags , :custom , :reverse_name ]","ref":"Flop.Schema.html#filterable/1"},{"type":"function","title":"Flop.Schema.get_field/2","doc":"Gets the field value from a struct. Resolves join fields and compound fields according to the config. # join_fields: [owner_name: {:owner, :name}] iex&gt; pet = % MyApp.Pet { name : &quot;George&quot; , owner : % MyApp.Owner { name : &quot;Carl&quot; } } iex&gt; Flop.Schema . get_field ( pet , :name ) &quot;George&quot; iex&gt; Flop.Schema . get_field ( pet , :owner_name ) &quot;Carl&quot; # compound_fields: [full_name: [:family_name, :given_name]] iex&gt; pet = % MyApp.Pet { given_name : &quot;George&quot; , family_name : &quot;Gooney&quot; } iex&gt; Flop.Schema . get_field ( pet , :full_name ) &quot;Gooney George&quot; For join fields, this function relies on the binding name in the schema config matching the field name for the association in the struct.","ref":"Flop.Schema.html#get_field/2"},{"type":"function","title":"Flop.Schema.max_limit/1","doc":"Returns the maximum limit of a schema. iex&gt; Flop.Schema . max_limit ( % MyApp.Pet { } ) 1000","ref":"Flop.Schema.html#max_limit/1"},{"type":"function","title":"Flop.Schema.pagination_types/1","doc":"Returns the allowed pagination types of a schema. iex&gt; Flop.Schema . pagination_types ( % MyApp.Fruit { } ) [ :first , :last , :offset ]","ref":"Flop.Schema.html#pagination_types/1"},{"type":"function","title":"Flop.Schema.sortable/1","doc":"Returns the sortable fields of a schema. iex&gt; Flop.Schema . sortable ( % MyApp.Pet { } ) [ :name , :age , :owner_name , :owner_age ]","ref":"Flop.Schema.html#sortable/1"},{"type":"type","title":"Flop.Schema.custom_field_option/0","doc":"Defines the options for a custom field. :filter (required) - A module/function/options tuple referencing a custom filter function. The function must take the Ecto query, the Flop.Filter struct, and the options from the tuple as arguments. :ecto_type - The Ecto type of the field. The filter operator and value validation is based on this option. :bindings - If the custom filter function requires certain named bindings to be present in the Ecto query, you can specify them here. These bindings will be conditionally added by Flop.with_named_bindings/4 if the filter is used. :operators - Defines which filter operators are allowed for this field. If omitted, all operators will be accepted. If both the :ecto_type and the :operators option are set, the :operators option takes precendence and only the filter value validation is based on the :ecto_type .","ref":"Flop.Schema.html#t:custom_field_option/0"},{"type":"type","title":"Flop.Schema.ecto_type/0","doc":"Either an Ecto type, or reference to the type of an existing schema field, or an adhoc Ecto.Enum. Examples You can pass any Ecto type: :string :integer Ecto.UUID {:parameterized, Ecto.Enum, Ecto.Enum.init(values: [:one, :two])} Or reference a schema field: {:from_schema, MyApp.Pet, :mood} Or build an adhoc Ecto.Enum: {:ecto_enum, [:one, :two]} (This has the same effect as the :parameterized example above.) {:ecto_enum, [one: 1, two: 2]} Note that if you make an Ecto.Enum type this way, the filter value will be cast as an atom. This means the field you filter on also needs to be an Ecto.Enum , or a custom type that is able to cast atoms. You cannot use this on a string field.","ref":"Flop.Schema.html#t:ecto_type/0"},{"type":"type","title":"Flop.Schema.join_field_option/0","doc":"Defines the options for a join field. :binding (required) - Any named binding :field (required) :ecto_type - The Ecto type of the field. The filter operator and value validation is based on this option. :path - This option is used by Flop.Schema.get_field/2 to retrieve the field value from a row. That function is also used by the default cursor functions in Flop.Cursor to determine the cursors. If the option is omitted, it defaults to [binding, field] .","ref":"Flop.Schema.html#t:join_field_option/0"},{"type":"type","title":"Flop.Schema.option/0","doc":"Defines the options that can be passed when deriving the Flop.Schema protocol. :filterable (required) - Defines the fields by which you can filter. You can reference fields from the Ecto schema, join fields, compound fields and custom fields here. Alias fields are not supported. :sortable (required) - Defines the fields by which you can sort. You can reference fields from the Ecto schema, join fields, and alias fields here. Custom fields and compound fields are not supported. :default_limit - Defines the default limit to apply if no limit , page_size , first or last parameter is set. :max_limit - Defines the maximum limit that can be set via parameters. :default_order - Defines the default order if no order parameters are set. :pagination_types - Defines which pagination types are allowed for this schema. :default_pagination_type - Defines the default pagination type to use if no pagination parameters are set. :join_fields - Defines fields on named bindings. :compound_fields - Defines groups of fields that can be filtered by combined, e.g. a family name plus a given name field. :custom_fields - Defines fields for custom fields for which you define your own filter functions. :alias_field - Defines fields that reference aliases defined with Ecto.Query.API.selected_as/2 .","ref":"Flop.Schema.html#t:option/0"},{"type":"type","title":"Flop.Schema.t/0","doc":"All the types that implement this protocol.","ref":"Flop.Schema.html#t:t/0"},{"type":"exception","title":"Flop.InvalidParamsError","doc":"An error that is raised if the parameter validation fails.","ref":"Flop.InvalidParamsError.html"},{"type":"function","title":"Flop.InvalidParamsError.message/1","doc":"Callback implementation for Exception.message/1 .","ref":"Flop.InvalidParamsError.html#message/1"},{"type":"type","title":"Flop.InvalidParamsError.t/0","doc":"","ref":"Flop.InvalidParamsError.html#t:t/0"},{"type":"extras","title":"Flop","doc":"Flop is an Elixir library designed to easily apply filtering, ordering, and pagination to your Ecto queries.","ref":"readme.html"},{"type":"extras","title":"Flop - Features","doc":"Offset-based pagination: Allows pagination through offset / limit or page / page_size parameters. Cursor-based pagination: Also known as key set pagination, provides a more efficient alternative to offset-based pagination. Compatible with Relay pagination arguments. Sorting: Applies sort parameters on multiple fields in any direction. Filtering: Allows complex data filtering using multiple conditions, operators, and fields. Parameter validation: Ensures the validity of provided parameters. Configurable filterable and sortable fields: Only applies parameters to the fields that were explicitly configured as filterable or sortable. Join fields: Allows the application of pagination, sort, and filter parameters on any named binding. Provides functions to help you to avoid unnecessary join clauses. Compound fields: Provides the ability to apply filter parameters on multiple string fields, for example for a full name filter. Custom fields: Provides an escape hatch for filters that Flop is not able to build on its own. Relay connection formatter: Formats the connection in Relay style, providing edges, nodes, and page info. UI helpers and URL builders through Flop Phoenix : Pagination, sortable tables and filter forms.","ref":"readme.html#features"},{"type":"extras","title":"Flop - Installation","doc":"To get started, add flop to your dependencies list in your project's mix.exs file: def deps do [ { :flop , &quot;~&gt; 0.21.0&quot; } ] end You can also configure a default repo for Flop by adding the following line to your config file: config :flop , repo : MyApp.Repo Instead of configuring Flop globally, you can also use a configuration module. Please refer to the Flop module documentation for more information.","ref":"readme.html#installation"},{"type":"extras","title":"Flop - Usage","doc":"Define sortable and filterable fields To define sortable and filterable fields in your Ecto schema, you can derive Flop.Schema . This step is optional but highly recommended, particularly when the parameters passed to Flop's functions are user-provided. Deriving Flop.Schema ensures that Flop applies filtering and sorting parameters only to the fields you've explicitly configured. defmodule MyApp.Pet do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age , :species ] } schema &quot;pets&quot; do field :name , :string field :age , :integer field :species , :string field :social_security_number , :string end end Besides sortable and filterable fields, Flop.Schema also allows the definition of join fields, compound fields, or custom fields. You can also set maximum or default limits, among other options. For a comprehensive list of available options, check the Flop.Schema documentation. Query data Use the Flop.validate_and_run/3 or Flop.validate_and_run!/3 function to both validate the parameters and fetch data from the database, and acquire pagination metadata in one operation. Here is an example of how you might use this in your code: defmodule MyApp.Pets do import Ecto.Query , warn : false alias Ecto.Changeset alias MyApp . { Pet , Repo } @spec list_pets ( map ) :: { :ok , { [ Pet . t ( ) ] , Flop.Meta . t ( ) } } | { :error , Flop.Meta . t ( ) } def list_pets ( params \\\\ %{ } ) do Flop . validate_and_run ( Pet , params , for : Pet ) end end The for option sets the Ecto schema for which you derived Flop.Schema . If you haven't derived Flop.Schema as described above, this option can be omitted. However, this is not recommended unless all parameters are generated internally and are guaranteed to be safe. On success, Flop.validate_and_run/3 returns an :ok tuple. The second element of this tuple is another tuple containing the fetched data and metadata. { :ok , { [ % Pet { } ] , % Flop.Meta { } } } You can learn more about the Flop.Meta struct in the module documentation . Alternatively, you may separate parameter validation and data fetching into different steps using the Flop.validate/2, Flop.validate!/2, and Flop.run/3 functions. This allows you to manipulate the validated parameters, to modify the query depending on the parameters, or to move the parameter validation to a different layer of your application. with { :ok , flop } &lt;- Flop . validate ( params , for : Pet ) do Flop . run ( Pet , flop , for : Pet ) end The aforementioned functions internally call the lower-level functions Flop.all/3 , Flop.meta/3 , and Flop.count/3 . If you have advanced requirements, you might prefer to use these functions directly. However, it's important to note that these lower-level functions do not validate the parameters. If parameters are generated based on user input, they should always be validated first using Flop.validate/2 or Flop.validate!/2 to ensure safe execution. The examples above assume that you configured a default repo. However, you can also pass the repo directly to the functions: Flop . validate_and_run ( Pet , flop , repo : MyApp.Repo ) Flop . all ( Pet , flop , repo : MyApp.Repo ) Flop . meta ( Pet , flop , repo : MyApp.Repo ) For more detailed information, refer the documentation .","ref":"readme.html#usage"},{"type":"extras","title":"Flop - Parameter format","doc":"The Flop library requires parameters to be provided in a specific format as a map. This map can be translated into a URL query parameter string, typically for use in a web framework like Phoenix. Pagination Offset / limit You can specify an offset to start from and a limit to the number of results. %{ offset : 20 , limit : 10 } This translates to the following query parameter string: ?offset=20&amp;limit=10 Page / page size You can specify the page number and the size of each page. %{ page : 2 , page_size : 10 } This translates to the following query parameter string: ?page=2&amp;page_size=10 Cursor You can fetch a specific number of results before or after a given cursor. %{ first : 10 , after : &quot;g3QAAAABZAACaWRiAAACDg==&quot; } %{ last : 10 , before : &quot;g3QAAAABZAACaWRiAAACDg==&quot; } These translate to the following query parameter strings: ?first=10&amp;after=g3QAAAABZAACaWRiAAACDg== ?last=10&amp;before=g3QAAAABZAACaWRiAAACDg== Ordering To sort the results, specify fields to order by and the direction of sorting for each field. %{ order_by : [ :name , :age ] , order_directions : [ :asc , :desc ] } This translates to the following query parameter string: ?order_by[]=name&amp;order_by[]=age&amp;order_directions[]=asc&amp;order_directions[]=desc Filters You can filter the results by providing a field, an operator, and a value. The operator is optional and defaults to == . Multiple filters are combined with a logical AND . At the moment, combining filters with OR is not supported. %{ filters : [ %{ field : :name , op : :ilike_and , value : &quot;Jane&quot; } ] } This translates to the following query parameter string: ?filters[0][field]=name&amp;filters[0][op]=ilike_and&amp;filters[0][value]=Jane Refer to the Flop.Filter documentation and Flop.t/0 type documentation for more details on using filters.","ref":"readme.html#parameter-format"},{"type":"extras","title":"Flop - Internal parameters","doc":"Flop is designed to manage parameters that come from the user side. While it is possible to alter those parameters and append extra filters upon receiving them, it is advisable to clearly differentiate parameters coming from outside and the parameters that your application adds internally. Consider the scenario where you need to scope a query based on the current user. In this case, it is better to create a separate function that introduces the necessary WHERE clauses: def list_pets ( %{ } = params , % User { } = current_user ) do Pet |&gt; scope ( current_user ) |&gt; Flop . validate_and_run ( params , for : Pet ) end defp scope ( q , % User { role : :admin } ) , do : q defp scope ( q , % User { id : user_id } ) , do : where ( q , user_id : ^ user_id ) If you need to add extra filters that are only used internally and aren't exposed to the user, you can pass them as a separate argument. This same argument can be used to override certain options depending on the context in which the function is called. def list_pets ( %{ } = args , opts \\\\ [ ] , % User { } = current_user ) do flop_opts = opts |&gt; Keyword . take ( [ :default_limit , :default_pagination_type , :pagination_types ] ) |&gt; Keyword . put ( :for , Pet ) Pet |&gt; scope ( current_user ) |&gt; apply_filters ( opts ) |&gt; Flop . validate_and_run ( flop , flop_opts ) end defp scope ( q , % User { role : :admin } ) , do : q defp scope ( q , % User { id : user_id } ) , do : where ( q , user_id : ^ user_id ) defp apply_filters ( q , opts ) do Enum . reduce ( opts , q , fn { :last_health_check , dt } , q -&gt; where ( q , [ p ] , p . last_health_check &lt; ^ dt ) { :reminder_service , bool } , q -&gt; where ( q , [ p ] , p . reminder_service == ^ bool ) _ , q -&gt; q end ) end With this approach, you maintain a clean separation between user-driven parameters and system-driven parameters, leading to more maintainable and less error-prone code.","ref":"readme.html#internal-parameters"},{"type":"extras","title":"Flop - Relay and Absinthe","doc":"The Flop.Relay module is useful if you are using absinthe with absinthe_relay , or if you simply need to adhere to the Relay cursor specification. This module provides functions that help transform query responses into a format compatible with Relay. Consider the scenario where you have defined node objects for owners and pets, along with a connection field for pets on the owner node object. node object ( :owner ) do field :name , non_null ( :string ) field :email , non_null ( :string ) connection field :pets , node_type : :pet do resolve &amp; MyAppWeb.Resolvers.Pet . list_pets / 2 end end node object ( :pet ) do field :name , non_null ( :string ) field :age , non_null ( :integer ) field :species , non_null ( :string ) end connection ( node_type : :pet ) Absinthe Relay will establish the arguments after , before , first and last on the pets field. These argument names align with those used by Flop, facilitating their application. Next, we'll define a list_pets_by_owner/2 function in the Pets context. defmodule MyApp.Pets do import Ecto.Query alias MyApp . { Owner , Pet , Repo } @spec list_pets_by_owner ( Owner . t ( ) , map ) :: { :ok , { [ Pet . t ( ) ] , Flop.Meta . t ( ) } } | { :error , Flop.Meta . t ( ) } def list_pets_by_owner ( % Owner { id : owner_id } , params \\\\ %{ } ) do Pet |&gt; where ( owner_id : ^ owner_id ) |&gt; Flop . validate_and_run ( params , for : Pet ) end end Now, within your resolver, you merely need to invoke the function and call Flop.Relay.connection_from_result/1 , which transforms the result into a tuple composed of the edges and the page_info, as required by absinthe_relay . defmodule MyAppWeb.Resolvers.Pet do alias MyApp . { Owner , Pet } def list_pets ( args , %{ source : % Owner { } = owner } = resolution ) do with { :ok , result } &lt;- Pets . list_pets_by_owner ( owner , args ) do { :ok , Flop.Relay . connection_from_result ( result ) } end end end In case you want to introduce additional filter arguments, you can employ Flop.nest_filters/3 to convert simple filter arguments into Flop filters, without necessitating API users to understand the Flop filter format. Let's add name and species filter arguments to the pets connection field. node object ( :owner ) do field :name , non_null ( :string ) field :email , non_null ( :string ) connection field :pets , node_type : :pet do arg :name , :string arg :species , :string resolve &amp; MyAppWeb.Resolvers.Pet . list_pets / 2 end end Assuming that these fields have been already configured as filterable with Flop.Schema , we can use Flop.nest_filters/3 to take the filter arguments and transform them into a list of Flop filters. defmodule MyAppWeb.Resolvers.Pet do alias MyApp . { Owner , Pet } def list_pets ( args , %{ source : % Owner { } = owner } = resolution ) do args = nest_filters ( args , [ :name , :species ] ) with { :ok , result } &lt;- Pets . list_pets_by_owner ( owner , args ) do { :ok , Flop.Relay . connection_from_result ( result ) } end end end Flop.nest_filters/3 uses the equality operator :== by default. You can override the default operator per field. args = nest_filters ( args , [ :name , :species ] , operators : %{ name : :ilike_and } )","ref":"readme.html#relay-and-absinthe"},{"type":"extras","title":"Flop - Flop Phoenix","doc":"Flop Phoenix is a companion library that provides Phoenix components for pagination, sortable tables, and filter forms, usable with both Phoenix LiveView and in dead views. It also defines helper functions to build URLs with Flop query parameters.","ref":"readme.html#flop-phoenix"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"Changelog - Unreleased","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"Changelog - [0.21.0] - 2023-07-02","doc":"Added Introduced operators as a new option for restricting acceptable operators for a custom field. Added bindings option for custom fields, allowing required named bindings to be added via Flop.with_named_bindings/4 . The ecto_type option on join and custom fields now supports references: {:from_schema, MySchema, :some_field} . The ecto_type option now supports a convenient syntax for adhoc enums: {:ecto_enum, [:one, :two]} . Improved documentation with added type definitions: Flop.Schema.option/0 , Flop.Schema.join_field_option/0 , Flop.Schema.custom_field_option/0 , and Flop.Schema.ecto_type/0 , describing options available when deriving the Flop.Schema protocol. Changed Breaking change: Filter values are now dynamically cast based on the field type and operator, instead of allowing any arbitrary filter value. This change ensures that invalid filter values cause validation errors instead of cast errors. The options for deriving the Flop.Schema protocol and for use Flop now undergo stricter validation with NimbleOptions . Flop.Cursor.encode/1 now explicitly sets the minor version option for :erlang.term_to_binary/2 to 2 , aligning with the new default in OTP 26. Before, this option was not set at all. Added a decoded_cursor field to the Flop struct. This field temporarily stores the decoded cursor between validation and querying and is discarded when generating the meta data. Deprecated The tuple syntax for defining join fields has been deprecated in favor of a keyword list. Fixed Resolved an issue where setting replace_invalid_params to true still caused validation errors for pagination and sorting parameters due to cast errors, instead of defaulting to valid parameters. Fixed the type specification for Flop.Filter.allowed_operators/1 . Upgrade notes The newly implemented dynamic casting of filter values could impact your code: Filter values failing to cast into the determined type will now yield a validation error or result in the removal of the invalid filter if the replace_invalid_params option is enabled. The value field of the Flop.Filter struct now holds the cast value instead of the original parameter value. For instance, while handling parameters generated via an HTML form with Flop, previously all filter values would be represented as strings in the struct. However, they may now be integers, DateTime structs, and so forth. Look out for this if you are directly reading or manipulating Flop.Filter structs. For join and custom fields, the type is determined with the ecto_type option. Previously, this option was only used for operator validation. Ensure the correct Ecto type is set. If the option is omitted, the filter values will continue to use their incoming format. Manual casting of filter values in a custom filter function is no longer required if the ecto_type option is set. If join fields point to Ecto.Enum fields, previously you could simply set ecto_type to string. This will continue to work if the filter value is passed as a string, but passing it as an atom will cause an error. Make sure to correctly reference the schema field ( {:from_schema, MySchema, :some_field} ) or directly pass the Enum values ( {:ecto_enum, [:one, :two} ). To enable Flop.Phoenix to build a query string for filter parameters, the filter value must be convertible into a string via to_string/1 . If ecto_type is set to a custom Ecto type that casts values into a struct, the String.Chars protocol must be implemented for that struct. Please review the newly added &quot;Ecto type option&quot; section in the Flop.Schema module documentation. Join field syntax If you are using tuples to define join fields when deriving Flop.Schema , update the configuration to use keyword lists instead: @derive { Flop.Schema, join_fields: [ - owner_name: {:owner, :name} + owner_name: [binding: :owner, field: :name] ] }","ref":"changelog.html#0-21-0-2023-07-02"},{"type":"extras","title":"Changelog - [0.20.3] - 2023-06-23","doc":"Changed Flop.count/3 will now wrap queries that have GROUP BY clauses in a subquery. Fixed Fixed cursor-based pagination on composite types.","ref":"changelog.html#0-20-3-2023-06-23"},{"type":"extras","title":"Changelog - [0.20.2] - 2023-06-09","doc":"Changed Added nutrition facts about use Flop and @derive Flop.Schema . The minimum Elixir version is now 1.11. Fixed Fixed a deprecation warning about Logger.warn/1 . Fixed a deprecation warning about passing an MFA to :with in cast_assoc/cast_embed introduced in Ecto 3.10.2.","ref":"changelog.html#0-20-2-2023-06-09"},{"type":"extras","title":"Changelog - [0.20.1] - 2023-05-19","doc":"Added Added the :count override option to Flop.count/3 . Changed The default_pagination_type can now be set in the schema. Fixed Don't raise function clause error in Flop.to_previous_cursor/1 and Flop.to_next_cursor/1 when the start cursor or end cursor are nil .","ref":"changelog.html#0-20-1-2023-05-19"},{"type":"extras","title":"Changelog - [0.20.0] - 2023-03-21","doc":"Added Added Flop.unnest_filters/3 as a reverse operation of Flop.nest_filters/3 after retrieving data from the database. Added Flop.Filter.fetch_value/2 , Flop.Filter.get_value/2 , Flop.Filter.put_value/4 , Flop.Filter.put_new_value/4 , Flop.Filter.pop_value/3 and Flop.Filter.pop_first_value/3 . Changed Several of the functions for manipulating lists of filters in the Flop.Filter module now accept lists of maps with atom keys, lists of maps with string keys, and indexed maps as produced by Phoenix HTML forms as argument. The empty and not_empty operators now treat empty maps as empty values on map fields and empty arrays as empty values on array fields. % and _ characters in filter values for the like , ilike and =~ operators are now escaped. Fixed Fixed an issue that caused filter conditions for like_and , like_or , ilike_and and ilike_or to be incorrectly combined when applied to compound fields.","ref":"changelog.html#0-20-0-2023-03-21"},{"type":"extras","title":"Changelog - [0.19.0] - 2023-01-15","doc":"Added Support for custom fields. These fields allow you to run custom filter functions for anything that cannot be expressed with Flop filters. Added Flop.with_named_bindings/4 for dynamically adding bindings needed for a Flop query. Added fetch , get , get_all , delete , delete_first , drop , new , take , pop , pop_first , put and put_new functions to Flop.Filter . Added Flop.Meta.with_errors/3 . Added ecto_type option to join fields. Added not_like and not_ilike filter operators. Added a cheatsheet for schema configuration. Added opts field to Flop.Meta struct. Changed Renamed Flop.bindings/3 to Flop.named_bindings/3 . Flop.Filter.allowed_operators/2 now tries to determine the Ecto type by reading the Flop field type from the schema module. This function is used during parameter validation, which means the validation step will be a bit stricter now. For join and custom fields, the Ecto type is determined via the new ecto_type option. If the option is not set, the function returns all operators as before. For compound fields, only the supported operators are returned.","ref":"changelog.html#0-19-0-2023-01-15"},{"type":"extras","title":"Changelog - [0.18.4] - 2022-11-17","doc":"Changed The :ilike_and , :ilike_or , :like_and and :like_or filter operators can now also be used with a list of strings as filter value.","ref":"changelog.html#0-18-4-2022-11-17"},{"type":"extras","title":"Changelog - [0.18.3] - 2022-10-27","doc":"Fixed default_pagination_type can be overridden by passing false now.","ref":"changelog.html#0-18-3-2022-10-27"},{"type":"extras","title":"Changelog - [0.18.2] - 2022-10-19","doc":"Fixed Flop.bindings/3 did not consider join fields that are used as part of a compound field.","ref":"changelog.html#0-18-2-2022-10-19"},{"type":"extras","title":"Changelog - [0.18.1] - 2022-10-14","doc":"Changed If the given map already has a :filters / &quot;filters&quot; key, Flop.nest_filters/3 will now merge the derived filters into the existing filters. If the existing filters are formatted as a map (as produced by an HTML form), they are converted to a list first. use Flop will now also compile validate/2 and validate!/2 functions that apply the options of your config module. Allow setting default_limit and max_limit to false , which removes the default/max limit without falling back to global options. Fixed Flop.bindings/3 was returning bindings for filters with nil values.","ref":"changelog.html#0-18-1-2022-10-14"},{"type":"extras","title":"Changelog - [0.18.0] - 2022-10-10","doc":"Added Added alias_fields option to Flop.Schema , which allows you to sort by field aliases defined with Ecto.Query.API.selected_as/2 . Added aliases/2 for getting the alias fields needed for a query. Added documentation example for filtering by calculated values. New option rename for Flop.map_to_filter_params/2 and Flop.nest_filters/3 . New option :replace_invalid_params . This option can be passed to the validate and validate_and_run functions or set in the global configuration or in a config module. Setting the value to true will cause Flop to replace invalid parameters with default values where possible or remove the parameter otherwise during the validation step, instead of returning validation errors. Changed Require ecto ~&gt; 3.9.0 . Flop.Schema does not raise an error anymore if a compound or join field is defined with the same name as a regular Ecto schema field. This was done so that you can add virtual fields with the same name. It is not possible to differentiate between non-virtual and virtual fields at compile time (at least I don't know how), so we cannot differentiate in the validation step. Flop applies a default limit of 50 and a max limit of 1000 now, unless other values are set. In offset/limit based pagination, the limit parameter is now required, in line with the other pagination types. If not set, it will fall back to a default limit.","ref":"changelog.html#0-18-0-2022-10-10"},{"type":"extras","title":"Changelog - [0.17.2] - 2022-10-03","doc":"Fixed Fixed an issue where the repo option was not read from a backend module.","ref":"changelog.html#0-17-2-2022-10-03"},{"type":"extras","title":"Changelog - [0.17.1] - 2022-10-02","doc":"Added Added a backend field to the Flop.Meta struct. Fixed Fixed an issue where the schema options were overridden by the backend module options.","ref":"changelog.html#0-17-1-2022-10-02"},{"type":"extras","title":"Changelog - [0.17.0] - 2022-08-26","doc":"Added Added the filter operators not_in and not_contains . Added examples for integration with Relay to the documentation. Added examples for the parameter format to the documentation. Changed Refactored the query builder. This does not affect users of the library, but makes the code base more readable and lays the groundwork for upcoming features. Added the :query_opts option to Flop callbacks to pass on options to the Ecto repo on query execution. If you are already using the :prefix option you now have to pass this through :query_opts . If you configured the Repo :prefix in the application config: config :flop, - prefix: &quot;some-prefix&quot; + query_opts: [prefix: &quot;some-prefix&quot;] If you set the :prefix when calling the Flop functions: - Flop.validate_and_run(Pet, params, prefix: &quot;some-prefix&quot;) + Flop.validate_and_run(Pet, params, query_opts: [prefix: &quot;some-prefix&quot;])","ref":"changelog.html#0-17-0-2022-08-26"},{"type":"extras","title":"Changelog - [0.16.1] - 2022-04-05","doc":"Fixed Wrong type spec for Flop.Schema.default_order/1 callback.","ref":"changelog.html#0-16-1-2022-04-05"},{"type":"extras","title":"Changelog - [0.16.0] - 2022-03-22","doc":"Added You can now define a configuration module with use Flop to set defaults instead of or in addition to the application configuration. This makes it easier to work with multiple Ecto repos. The new function Flop.bindings/3 returns the necessary bindings for a given Flop query. You can use it in case you want to optimize your queries by only joining tables that are actually needed. Added a count_query option to override the count query used by Flop.run/3 , Flop.validate_and_run/3 and Flop.validate_and_run!/3 . You can get a list of allowed operators for a given Ecto type or a given schema field with Flop.Filter.allowed_operators/1 and Flop.Filter.allowed_operators/2 now. Changed Breaking: The :empty and :not_empty filters now require a boolean value. If no value is passed, the filter is ignored, just as it is handled for all other filter operators. This change was necessary to make the integration with filter forms (checkboxes) easier. Breaking: The default order needs to be passed as a map now when deriving Flop.Schema . The previous implementation already converted the two separate configuration keys to a map. This meant that the configuration passed when deriving Flop.Schema had a different format from the one you had to pass when overriding the default order with the opts . With this change, the configuration format is the same everywhere. A compile time exception is raised if you are still using the old format, guiding you in the update. It is now validated that the filter operator matches the field type. The compile time validation of the options passed when deriving Flop.Schema has been improved. Allow passing page as string to Flop.set_page/2 . Allow passing offset as string to Flop.set_offset/2 .","ref":"changelog.html#0-16-0-2022-03-22"},{"type":"extras","title":"Changelog - [0.15.0] - 2021-11-14","doc":"Added Add Flop.reset_filters/1 and Flop.reset_order/1 . Add Flop.current_order/2 to retrieve the order of a given field. Add Flop.to_next_page/2 and Flop.to_previous_page/1 . Add Flop.set_cursor/2 , Flop.to_next_cursor/1 and Flop.to_previous_cursor/1 . Add Flop.set_offset/2 , Flop.to_previous_offset/1 , Flop.to_next_offset_2 and Flop.reset_cursors/2 . Add Flop.nest_filters/3 for converting filters between a key/value map and a list of Flop.Filter parameters. You can now set the default_pagination_type option, which forces a certain set of parameters when defaults are applied and the pagination type cannot be determined from the given parameters. Add optional default argument to get_option . Add pagination option. If set to true , pagination parameters are not cast. Changed Flop.map_to_filter_params/2 returns maps with string keys if the original map has string keys now. The has_previous_page? value of the Flop.Meta struct is now always true if first is used with after . has_next_page? is always true when last is used with before . push_order/2 resets the :after and :before parameters now, since the cursors depend on the order. validate_and_run/3 and validate_and_run!/3 pass all given options to the validate functions now, allowing you to override defaults set in the schema. If the pagination_types option is used, parameters for other pagination types will not be cast now instead of casting them and returning validation errors. Removed Remove Flop.Cursor.get_cursor_from_map/2 . Use Flop.Cursor.get_cursor_from_node/2 instead.","ref":"changelog.html#0-15-0-2021-11-14"},{"type":"extras","title":"Changelog - [0.14.0] - 2021-11-08","doc":"Added Add :contains operator. Add Flop.map_to_filter_params/2 . Changed Flop.validate/2 and Flop.validate_and_run/3 return {:error, Flop.Meta.t} instead of {:error, Ecto.Changeset.t} now. The Meta struct has the new fields :errors and :params , which are set when validation errors occur. This accompanies the changes in Flop.Phoenix , which include the implementation of the Phoenix.HTML.FormData protocol for the Flop.Meta struct. Flop.validate!/2 and Flop.validate_and_run!/3 raise a Flop.InvalidParamsError instead of an Ecto.InvalidChangesetError now. Add :schema key to Flop.Meta . This field points to the schema module set by passing the :for option. Minimum Ecto version changed to 3.5. Replace Operator and OrderDirection custom Ecto types with Ecto.Enum. Update Flop.Meta struct default values for the fields :flop , :has_next_page? and :has_previous_page? .","ref":"changelog.html#0-14-0-2021-11-08"},{"type":"extras","title":"Changelog - [0.13.2] - 2021-10-16","doc":"Fixed Fix error when sorting by a compound field that consists of at least one join field. Fix import conflict when importing Ecto.Changeset in a module that derives Flop.Schema and configures a compound field.","ref":"changelog.html#0-13-2-2021-10-16"},{"type":"extras","title":"Changelog - [0.13.1] - 2021-08-23","doc":"Fixed Wrong type spec for cursor_dynamic/3 callback.","ref":"changelog.html#0-13-1-2021-08-23"},{"type":"extras","title":"Changelog - [0.13.0] - 2021-08-22","doc":"Added Support ordering by join fields. Support ordering by compound fields. Support join fields as cursor fields. New function Flop.Schema.get_field/2 . Flop.Cursor.get_cursor_from_edge/2 and Flop.Cursor.get_cursor_from_node/2 can get cursor values from join and compound fields now. Changed To get the pagination cursor value from a join field, Flop needs to know how to access the field value from the returned struct or map. The configuration format for join fields has been changed to allow specifying the path to the nested field. Before: @derive { Flop.Schema , join_fields : [ owner_name : { :owner , :name } ] } After: @derive { Flop.Schema , join_fields : [ owner_name : [ binding : :owner , field : :name , path : [ :owner , :name ] ] ] } The :path is optional and inferred from the :binding and :field options, if omitted. The old configuration format is still accepted. All of these settings are equivalent: [ owner_name : { :owner , :name } ] [ owner_name : [ binding : :owner , field : :name ] ] [ owner_name : [ binding : :owner , field : :name , path : [ :owner , :name ] ] ] Fixed Cursor pagination failed when one of the cursor field values was nil .","ref":"changelog.html#0-13-0-2021-08-22"},{"type":"extras","title":"Changelog - [0.12.0] - 2021-08-11","doc":"Added Allow to define join fields in Flop.Schema . Allow to define compound fields in Flop.Schema . Support filtering by join fields. Support filtering by compound fields. New filter operator empty . New filter operator not_empty . New function Flop.set_page/2 . Changed Rename option get_cursor_value_func to cursor_value_func . Silently ignore filters with nil value for the field or the value instead of raising an ArgumentError . Allow passing a string as the second argument to Flop.push_order/2 .","ref":"changelog.html#0-12-0-2021-08-11"},{"type":"extras","title":"Changelog - [0.11.0] - 2021-06-13","doc":"Added New functions Flop.Cursor.get_cursor_from_node/2 and Flop.Cursor.get_cursor_from_edge/2 . New function Flop.get_option/2 . Support Ecto prefixes. Changed Use Flop.Cursor.get_cursor_from_node/2 as default for the :get_cursor_value_func option. Flop.Relay.edges_from_result/2 can now handle nil instead of a map as edge information in a query result. Deprecated Deprecate Flop.Cursor.get_cursor_from_map/2 . Use Flop.Cursor.get_cursor_from_node/2 instead.","ref":"changelog.html#0-11-0-2021-06-13"},{"type":"extras","title":"Changelog - [0.10.0] - 2021-05-03","doc":"Added Add function Flop.push_order/2 for updating the order_by and order_directions values of a Flop struct.","ref":"changelog.html#0-10-0-2021-05-03"},{"type":"extras","title":"Changelog - [0.9.1] - 2020-10-21","doc":"Fixed Fixed type spec of Flop.Schema.default_order/1 .","ref":"changelog.html#0-9-1-2020-10-21"},{"type":"extras","title":"Changelog - [0.9.0] - 2020-10-16","doc":"Added Add like , like_and , like_or , ilike , ilike_and and ilike_or filter operators. Add option to disable pagination types globally, for a schema or locally. Add options to disable ordering or filtering. Allow global configuration of get_cursor_value_func , max_limit and default_limit . Add Flop.option type, improve documentation of available options. Add Flop.Cursor.decode!/1 . Changed Refactored the parameter validation. Default limits are now applied to all pagination types. Added validation for the after / before cursor values. Flop.Cursor.decode/1 returns :ok tuple or :error now instead of raising an error if the cursor is invalid. Flop.Cursor.decode/1 returns an error if the decoded cursor value is not a map with atom keys. Improved documentation.","ref":"changelog.html#0-9-0-2020-10-16"},{"type":"extras","title":"Changelog - [0.8.4] - 2020-10-14","doc":"Fixed Default limit was overriding first / last parameters when building query.","ref":"changelog.html#0-8-4-2020-10-14"},{"type":"extras","title":"Changelog - [0.8.3] - 2020-10-14","doc":"Fixed Cursor-based pagination: has_next_page? was set when querying with last based on before being set. Likewise, has_previous_page? was set when querying with first based on after being set. Both assumptions are wrong. In both cases, the values are always set to false now.","ref":"changelog.html#0-8-3-2020-10-14"},{"type":"extras","title":"Changelog - [0.8.2] - 2020-10-08","doc":"Changed Order directions are not restricted anymore for cursor-based pagination. Fixed Query for cursor-based pagination returned wrong results when using more than one cursor field. Query for cursor-based pagination returned wrong results when using last / before .","ref":"changelog.html#0-8-2-2020-10-08"},{"type":"extras","title":"Changelog - [0.8.1] - 2020-10-07","doc":"Changed Allow structs in cursor values.","ref":"changelog.html#0-8-1-2020-10-07"},{"type":"extras","title":"Changelog - [0.8.0] - 2020-10-07","doc":"Added Support for cursor-based pagination. Thanks to @bunker-inspector. Add functions to turn query results into Relay connection format when using cursor-based pagination.","ref":"changelog.html#0-8-0-2020-10-07"},{"type":"extras","title":"Changelog - [0.7.1] - 2020-09-04","doc":"Fixed Calculation of has_next_page? was wrong.","ref":"changelog.html#0-7-1-2020-09-04"},{"type":"extras","title":"Changelog - [0.7.0] - 2020-08-04","doc":"Added Flop.Schema now allows to set a default sort order. Changed Passing a limit without an offset will now set the offset to 0. Passing a page size without a page will now set the page to 1.","ref":"changelog.html#0-7-0-2020-08-04"},{"type":"extras","title":"Changelog - [0.6.1] - 2020-06-17","doc":"Changed Add Flop to Meta struct. Fixed Type Flop.Filter.op didn't include all operators.","ref":"changelog.html#0-6-1-2020-06-17"},{"type":"extras","title":"Changelog - [0.6.0] - 2020-06-14","doc":"Added New struct Flop.Meta . New function Flop.all/3 . New function Flop.count/3 . New function Flop.meta/3 . New function Flop.run/3 . New function Flop.validate_and_run/3 . New function Flop.validate_and_run!/3 .","ref":"changelog.html#0-6-0-2020-06-14"},{"type":"extras","title":"Changelog - [0.5.0] - 2020-05-28","doc":"Added New function Flop.validate!/2 . New filter operator :in . Fixed Filter validation was using sortable fields instead of filterable fields.","ref":"changelog.html#0-5-0-2020-05-28"},{"type":"extras","title":"Changelog - [0.4.0] - 2020-05-27","doc":"Added Added =~ filter operator. Fixed Query function wasn't generating valid where clauses for filters.","ref":"changelog.html#0-4-0-2020-05-27"},{"type":"extras","title":"Changelog - [0.3.0] - 2020-05-22","doc":"Added Added a default_limit option to Flop.Schema .","ref":"changelog.html#0-3-0-2020-05-22"},{"type":"extras","title":"Changelog - [0.2.0] - 2020-05-20","doc":"Added Added a max_limit option to Flop.Schema . When set, Flop validates that the limit and page_size parameters don't exceed the configured max limit.","ref":"changelog.html#0-2-0-2020-05-20"},{"type":"extras","title":"Changelog - [0.1.0] - 2019-10-19","doc":"initial release","ref":"changelog.html#0-1-0-2019-10-19"},{"type":"extras","title":"Schema configuration","doc":"","ref":"schema.html"},{"type":"extras","title":"Schema configuration - Basics","doc":"Minimal configuration defmodule Pet do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] } schema &quot;pets&quot; do field :name , :string field :age , :integer field :species , :string end end Options Limit @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , max_limit : 100 , default_limit : 50 } Order @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , default_order : %{ order_by : [ :name , :age ] , order_directions : [ :asc , :desc ] } } Pagination types @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , pagination_types : [ :first , :last ] , default_pagination_type : :first }","ref":"schema.html#basics"},{"type":"extras","title":"Schema configuration - Alias fields","doc":"Schema defmodule Owner do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name ] , sortable : [ :name , :pet_count ] , alias_fields : [ :pet_count ] } schema &quot;owners&quot; do field :name , :string has_many :pets , Pet end end Query params = %{ order_by : [ :pet_count ] } Owner |&gt; join ( :left , [ o ] , p in assoc ( o , :pets ) , as : :pets ) |&gt; group_by ( [ o ] , o . id ) |&gt; select ( [ o , pets : p ] , { o . id , p . id |&gt; count ( ) |&gt; selected_as ( :pet_count ) } ) |&gt; Flop . validate_and_run ( params , for : Owner )","ref":"schema.html#alias-fields"},{"type":"extras","title":"Schema configuration - Compound fields","doc":"Schema defmodule User do use Ecto.Schema @derive { Flop.Schema , filterable : [ :full_name ] , sortable : [ :full_name ] , compound_fields : [ full_name : [ :family_name , :given_name ] ] } schema &quot;users&quot; do field :family_name , :string field :given_name , :string end end Query params = %{ filters : [ %{ field : :full_name , op : :ilike_and , value : &quot;pea&quot; } ] } Flop . validate_and_run ( User , params , for : Owner )","ref":"schema.html#compound-fields"},{"type":"extras","title":"Schema configuration - Join fields","doc":"Schema Owner defmodule Owner do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :pet_age ] , sortable : [ :name ] , join_fields : [ pet_age : [ binding : :pets , field : :age , ecto_type : :integer ] ] } schema &quot;owners&quot; do field :name , :string has_many :pets , Pet end end Pet defmodule Pet do use Ecto.Schema schema &quot;pets&quot; do field :age , :integer end end Query Only filtering or sorting params = %{ filters : [ %{ field : :pet_age , op : :== , value : 8 } ] } Owner |&gt; join ( [ o ] , p in assoc ( o , :pets ) , as : :pets ) |&gt; Flop . validate_and_run ( params , for : Pet ) With preload Owner |&gt; join ( [ o ] , p in assoc ( o , :pets ) , as : :pets ) |&gt; preload ( [ pets : p ] , pets : p ) |&gt; Flop . validate_and_run ( params , for : Pet )","ref":"schema.html#join-fields"},{"type":"extras","title":"Schema configuration - Join field for nested association","doc":"Schema Owner defmodule Owner do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :toy_description ] , sortable : [ :name ] , join_fields : [ pet_age : [ binding : :toys , field : :description , ecto_type : :string , # only needed with cursor pagination when sorting # by the join field, so that Flop can find the # cursor value path : [ :pets , :toys ] ] ] } schema &quot;owners&quot; do field :name , :string has_many :pets , Pet end end Pet defmodule Pet do use Ecto.Schema schema &quot;pets&quot; do field :age , :integer has_many :toys , Toy end end Toy defmodule Toy do use Ecto.Schema schema &quot;toys&quot; do field :description , :string end end Query with preload params = %{ order_by : [ :toy_description ] } Owner |&gt; join ( [ o ] , p in assoc ( o , :pets ) , as : :pets ) |&gt; join ( [ pets : p ] , t in assoc ( p , :toys ) , as : :toys ) |&gt; preload ( [ pets : p , toys : t ] , pets : { p , toys : t } ) |&gt; Flop . validate_and_run ( params , for : Owner )","ref":"schema.html#join-field-for-nested-association"},{"type":"extras","title":"Schema configuration - Join field for subquery","doc":"Schema Owner defmodule Owner do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name ] , sortable : [ :name , :pet_count ] , join_fields : [ pet_count : [ binding : :pet_count , field : :count ] ] } schema &quot;owners&quot; do field :name , :string has_many :pets , Pet end end Pet defmodule Pet do use Ecto.Schema schema &quot;pets&quot; do field :age , :integer end end Query params = %{ filters : [ %{ field : :pet_count , op : :&gt; , value : 2 } ] } pet_count_query = Pet |&gt; where ( [ p ] , parent_as ( :owner ) . id == p . owner_id ) |&gt; select ( [ p ] , %{ count : count ( p ) } ) q = Owner |&gt; from ( as : :owner ) |&gt; join ( :inner_lateral , [ o ] , p in subquery ( pet_count_query ) , as : :pet_count ) |&gt; Flop . validate_and_run ( params , for : Owner )","ref":"schema.html#join-field-for-subquery"},{"type":"extras","title":"Schema configuration - Custom fields","doc":"Schema defmodule Pet do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :human_age ] , sortable : [ :name ] , custom_fields : [ human_age : [ filter : { CustomFilters , :human_age , [ ] } , ecto_type : :integer ] ] } schema &quot;pets&quot; do field :name , :string field :age , :integer end end Custom filter function defmodule CustomFilters do import Ecto.Query def human_age ( q , % Flop.Filter { value : value , op : op } , _ ) do case Ecto.Type . cast ( :integer , value ) do { :ok , human_years } -&gt; value_in_dog_years = round ( human_years / 7 ) case op do :== -&gt; where ( q , [ p ] , p == ^ value_in_dog_years ) :!= -&gt; where ( q , [ p ] , p != ^ value_in_dog_years ) :&gt; -&gt; where ( q , [ p ] , p &gt; ^ value_in_dog_years ) :&lt; -&gt; where ( q , [ p ] , p &lt; ^ value_in_dog_years ) :&gt;= -&gt; where ( q , [ p ] , p &gt;= ^ value_in_dog_years ) :&lt;= -&gt; where ( q , [ p ] , p &lt;= ^ value_in_dog_years ) end :error -&gt; # cannot cast filter value, ignore q end end end Query params = %{ filters : [ %{ field : :human_age , op : :== , value : 30 } ] } Flop . validate_and_run ( Pet , params , for : Pet )","ref":"schema.html#custom-fields"}]